# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import WarpUtils


def test_WarpUtils_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='--in=%s',
            extensions=None,
            mandatory=True,
        ),
        knot_space=dict(argstr='--knotspace=%d,%d,%d', ),
        out_file=dict(
            argstr='--out=%s',
            extensions=None,
            name_source=['in_file'],
            output_name='out_file',
            position=-1,
        ),
        out_format=dict(argstr='--outformat=%s', ),
        out_jacobian=dict(
            argstr='--jac=%s',
            extensions=None,
        ),
        output_type=dict(),
        reference=dict(
            argstr='--ref=%s',
            extensions=None,
            mandatory=True,
        ),
        warp_resolution=dict(argstr='--warpres=%0.4f,%0.4f,%0.4f', ),
        with_affine=dict(argstr='--withaff', ),
        write_jacobian=dict(
            mandatory=True,
            usedefault=True,
        ),
    )
    inputs = WarpUtils.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_WarpUtils_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_jacobian=dict(extensions=None, ),
    )
    outputs = WarpUtils.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import ApplyWarp


def test_ApplyWarp_inputs():
    input_map = dict(
        abswarp=dict(
            argstr='--abs',
            xor=['relwarp'],
        ),
        args=dict(argstr='%s', ),
        datatype=dict(argstr='--datatype=%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        field_file=dict(
            argstr='--warp=%s',
            extensions=None,
        ),
        in_file=dict(
            argstr='--in=%s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        interp=dict(
            argstr='--interp=%s',
            position=-2,
        ),
        mask_file=dict(
            argstr='--mask=%s',
            extensions=None,
        ),
        out_file=dict(
            argstr='--out=%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            position=2,
        ),
        output_type=dict(),
        postmat=dict(
            argstr='--postmat=%s',
            extensions=None,
        ),
        premat=dict(
            argstr='--premat=%s',
            extensions=None,
        ),
        ref_file=dict(
            argstr='--ref=%s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        relwarp=dict(
            argstr='--rel',
            position=-1,
            xor=['abswarp'],
        ),
        superlevel=dict(argstr='--superlevel=%s', ),
        supersample=dict(argstr='--super', ),
    )
    inputs = ApplyWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ApplyWarp_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = ApplyWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Remlfit


def test_Remlfit_inputs():
    input_map = dict(
        STATmask=dict(
            argstr='-STATmask %s',
            extensions=None,
        ),
        addbase=dict(
            argstr='-addbase %s',
            copyfile=False,
            sep=' ',
        ),
        args=dict(argstr='%s', ),
        automask=dict(
            argstr='-automask',
            usedefault=True,
        ),
        dsort=dict(
            argstr='-dsort %s',
            copyfile=False,
            extensions=None,
        ),
        dsort_nods=dict(
            argstr='-dsort_nods',
            requires=['dsort'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        errts_file=dict(
            argstr='-Rerrts %s',
            extensions=None,
        ),
        fitts_file=dict(
            argstr='-Rfitts %s',
            extensions=None,
        ),
        fout=dict(argstr='-fout', ),
        glt_file=dict(
            argstr='-Rglt %s',
            extensions=None,
        ),
        gltsym=dict(argstr='-gltsym "%s" %s...', ),
        goforit=dict(argstr='-GOFORIT', ),
        in_files=dict(
            argstr='-input "%s"',
            copyfile=False,
            mandatory=True,
            sep=' ',
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        matim=dict(
            argstr='-matim %s',
            extensions=None,
            xor=['matrix'],
        ),
        matrix=dict(
            argstr='-matrix %s',
            extensions=None,
            mandatory=True,
        ),
        nobout=dict(argstr='-nobout', ),
        nodmbase=dict(
            argstr='-nodmbase',
            requires=['addbase', 'dsort'],
        ),
        nofdr=dict(argstr='-noFDR', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        obeta=dict(
            argstr='-Obeta %s',
            extensions=None,
        ),
        obuck=dict(
            argstr='-Obuck %s',
            extensions=None,
        ),
        oerrts=dict(
            argstr='-Oerrts %s',
            extensions=None,
        ),
        ofitts=dict(
            argstr='-Ofitts %s',
            extensions=None,
        ),
        oglt=dict(
            argstr='-Oglt %s',
            extensions=None,
        ),
        out_file=dict(
            argstr='-Rbuck %s',
            extensions=None,
        ),
        outputtype=dict(),
        ovar=dict(
            argstr='-Ovar %s',
            extensions=None,
        ),
        polort=dict(
            argstr='-polort %d',
            xor=['matrix'],
        ),
        quiet=dict(argstr='-quiet', ),
        rbeta_file=dict(
            argstr='-Rbeta %s',
            extensions=None,
        ),
        rout=dict(argstr='-rout', ),
        slibase=dict(argstr='-slibase %s', ),
        slibase_sm=dict(argstr='-slibase_sm %s', ),
        tout=dict(argstr='-tout', ),
        usetemp=dict(argstr='-usetemp', ),
        var_file=dict(
            argstr='-Rvar %s',
            extensions=None,
        ),
        verb=dict(argstr='-verb', ),
        wherr_file=dict(
            argstr='-Rwherr %s',
            extensions=None,
        ),
    )
    inputs = Remlfit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Remlfit_outputs():
    output_map = dict(
        errts_file=dict(extensions=None, ),
        fitts_file=dict(extensions=None, ),
        glt_file=dict(extensions=None, ),
        obeta=dict(extensions=None, ),
        obuck=dict(extensions=None, ),
        oerrts=dict(extensions=None, ),
        ofitts=dict(extensions=None, ),
        oglt=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        ovar=dict(extensions=None, ),
        rbeta_file=dict(extensions=None, ),
        var_file=dict(extensions=None, ),
        wherr_file=dict(extensions=None, ),
    )
    outputs = Remlfit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Parallel workflow execution via LSF
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
import re
from time import sleep

from ... import logging
from ...interfaces.base import CommandLine
from .base import SGELikeBatchManagerBase, logger
iflogger = logging.getLogger('nipype.interface')


class LSFPlugin(SGELikeBatchManagerBase):
    """Execute using LSF Cluster Submission

    The plugin_args input to run can be used to control the LSF execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - bsub_args : arguments to be prepended to the job execution script in the
                  bsub call

    """

    def __init__(self, **kwargs):
        template = """
#$ -S /bin/sh
        """
        self._retry_timeout = 2
        self._max_tries = 2
        self._bsub_args = ''
        if 'plugin_args' in kwargs and kwargs['plugin_args']:
            if 'retry_timeout' in kwargs['plugin_args']:
                self._retry_timeout = kwargs['plugin_args']['retry_timeout']
            if 'max_tries' in kwargs['plugin_args']:
                self._max_tries = kwargs['plugin_args']['max_tries']
            if 'bsub_args' in kwargs['plugin_args']:
                self._bsub_args = kwargs['plugin_args']['bsub_args']
        super(LSFPlugin, self).__init__(template, **kwargs)

    def _is_pending(self, taskid):
        """LSF lists a status of 'PEND' when a job has been submitted but is
        waiting to be picked up, and 'RUN' when it is actively being processed.
        But _is_pending should return True until a job has finished and is
        ready to be checked for completeness. So return True if status is
        either 'PEND' or 'RUN'"""
        cmd = CommandLine(
            'bjobs', resource_monitor=False, terminal_output='allatonce')
        cmd.inputs.args = '%d' % taskid
        # check lsf task
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName('CRITICAL'))
        result = cmd.run(ignore_exception=True)
        iflogger.setLevel(oldlevel)
        # logger.debug(result.runtime.stdout)
        if 'DONE' in result.runtime.stdout or 'EXIT' in result.runtime.stdout:
            return False
        else:
            return True

    def _submit_batchtask(self, scriptfile, node):
        cmd = CommandLine(
            'bsub',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        bsubargs = ''
        if self._bsub_args:
            bsubargs = self._bsub_args
        if 'bsub_args' in node.plugin_args:
            if 'overwrite' in node.plugin_args and\
               node.plugin_args['overwrite']:
                bsubargs = node.plugin_args['bsub_args']
            else:
                bsubargs += (" " + node.plugin_args['bsub_args'])
        if '-o' not in bsubargs:  # -o outfile
            bsubargs = '%s -o %s' % (bsubargs, scriptfile + ".log")
        if '-e' not in bsubargs:
            # -e error file
            bsubargs = '%s -e %s' % (bsubargs, scriptfile + ".log")
        if node._hierarchy:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._hierarchy,
                                node._id))
        else:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._id))
        jobnameitems = jobname.split('.')
        jobnameitems.reverse()
        jobname = '.'.join(jobnameitems)
        cmd.inputs.args = '%s -J %s sh %s' % (bsubargs, jobname,
                                              scriptfile)  # -J job_name_spec
        logger.debug('bsub ' + cmd.inputs.args)
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName('CRITICAL'))
        tries = 0
        while True:
            try:
                result = cmd.run()
            except Exception as e:
                if tries < self._max_tries:
                    tries += 1
                    sleep(
                        self._retry_timeout)  # sleep 2 seconds and try again.
                else:
                    iflogger.setLevel(oldlevel)
                    raise RuntimeError('\n'.join((('Could not submit lsf task'
                                                   ' for node %s') % node._id,
                                                  str(e))))
            else:
                break
        iflogger.setLevel(oldlevel)
        # retrieve lsf taskid
        match = re.search(r'<(\d*)>', result.runtime.stdout)
        if match:
            taskid = int(match.groups()[0])
        else:
            raise IOError("Can't parse submission job output id: %s" %
                          result.runtime.stdout)
        self._pending[taskid] = node.output_dir()
        logger.debug('submitted lsf task: %d for node %s' % (taskid, node._id))
        return taskid



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class dtiaverageInputSpec(CommandLineInputSpec):
    inputs = InputMultiPath(
        File(exists=True),
        desc="List of all the tensor fields to be averaged",
        argstr="--inputs %s...")
    tensor_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Averaged tensor volume",
        argstr="--tensor_output %s")
    DTI_double = traits.Bool(
        desc=
        "Tensor components are saved as doubles (cannot be visualized in Slicer)",
        argstr="--DTI_double ")
    verbose = traits.Bool(desc="produce verbose output", argstr="--verbose ")


class dtiaverageOutputSpec(TraitedSpec):
    tensor_output = File(desc="Averaged tensor volume", exists=True)


class dtiaverage(SEMLikeCommandLine):
    """title: DTIAverage (DTIProcess)

category: Diffusion.Diffusion Tensor Images.CommandLineOnly

description: dtiaverage is a program that allows to compute the average of an arbitrary number of tensor fields (listed after the --inputs option) This program is used in our pipeline as the last step of the atlas building processing. When all the tensor fields have been deformed in the same space, to create the average tensor field (--tensor_output) we use dtiaverage.
 Several average method can be used (specified by the --method option): euclidian, log-euclidian and pga. The default being euclidian.

version: 1.0.0

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/Nightly/Extensions/DTIProcess

license: Copyright (c)  Casey Goodlett. All rights reserved.
    See http://www.ia.unc.edu/dev/Copyright.htm for details.
    This software is distributed WITHOUT ANY WARRANTY; without even
    the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the above copyright notices for more information.

contributor: Casey Goodlett

"""

    input_spec = dtiaverageInputSpec
    output_spec = dtiaverageOutputSpec
    _cmd = " dtiaverage "
    _outputs_filenames = {'tensor_output': 'tensor_output.nii'}
    _redirect_x = False


class dtiestimInputSpec(CommandLineInputSpec):
    dwi_image = File(
        desc="DWI image volume (required)",
        exists=True,
        argstr="--dwi_image %s")
    tensor_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Tensor OutputImage",
        argstr="--tensor_output %s")
    B0 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Baseline image, average of all baseline images",
        argstr="--B0 %s")
    idwi = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "idwi output image. Image with isotropic diffusion-weighted information = geometric mean of diffusion images",
        argstr="--idwi %s")
    B0_mask_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "B0 mask used for the estimation. B0 thresholded either with the -t option value or the automatic OTSU value",
        argstr="--B0_mask_output %s")
    brain_mask = File(
        desc=
        "Brain mask.  Image where for every voxel == 0 the tensors are not estimated. Be aware that in addition a threshold based masking will be performed by default. If such an additional threshold masking is NOT desired, then use option -t 0.",
        exists=True,
        argstr="--brain_mask %s")
    bad_region_mask = File(
        desc=
        "Bad region mask.  Image where for every voxel > 0 the tensors are not estimated",
        exists=True,
        argstr="--bad_region_mask %s")
    method = traits.Enum(
        "lls",
        "wls",
        "nls",
        "ml",
        desc=
        "Esitmation method (lls:linear least squares, wls:weighted least squares, nls:non-linear least squares, ml:maximum likelihood)",
        argstr="--method %s")
    correction = traits.Enum(
        "none",
        "zero",
        "abs",
        "nearest",
        desc=
        "Correct the tensors if computed tensor is not semi-definite positive",
        argstr="--correction %s")
    threshold = traits.Int(
        desc=
        "Baseline threshold for estimation. If not specified calculated using an OTSU threshold on the baseline image.",
        argstr="--threshold %d")
    weight_iterations = traits.Int(
        desc=
        "Number of iterations to recaluate weightings from tensor estimate",
        argstr="--weight_iterations %d")
    step = traits.Float(
        desc="Gradient descent step size (for nls and ml methods)",
        argstr="--step %f")
    sigma = traits.Float(argstr="--sigma %f")
    DTI_double = traits.Bool(
        desc=
        "Tensor components are saved as doubles (cannot be visualized in Slicer)",
        argstr="--DTI_double ")
    verbose = traits.Bool(desc="produce verbose output", argstr="--verbose ")
    defaultTensor = InputMultiPath(
        traits.Float,
        desc=
        "Default tensor used if estimated tensor is below a given threshold",
        sep=",",
        argstr="--defaultTensor %s")
    shiftNeg = traits.Bool(
        desc=
        "Shift eigenvalues so all are positive (accounts for bad tensors related to noise or acquisition error). This is the same option as the one available in DWIToDTIEstimation in Slicer (but instead of just adding the minimum eigenvalue to all the eigenvalues if it is smaller than 0, we use a coefficient to have stictly positive eigenvalues",
        argstr="--shiftNeg ")
    shiftNegCoeff = traits.Float(
        desc=
        "Shift eigenvalues so all are positive (accounts for bad tensors related to noise or acquisition error). Instead of just adding the minimum eigenvalue to all the eigenvalues if it is smaller than 0, we use a coefficient to have stictly positive eigenvalues. Coefficient must be between 1.0 and 1.001 (included).",
        argstr="--shiftNegCoeff %f")


class dtiestimOutputSpec(TraitedSpec):
    tensor_output = File(desc="Tensor OutputImage", exists=True)
    B0 = File(
        desc="Baseline image, average of all baseline images", exists=True)
    idwi = File(
        desc=
        "idwi output image. Image with isotropic diffusion-weighted information = geometric mean of diffusion images",
        exists=True)
    B0_mask_output = File(
        desc=
        "B0 mask used for the estimation. B0 thresholded either with the -t option value or the automatic OTSU value",
        exists=True)


class dtiestim(SEMLikeCommandLine):
    """title: DTIEstim (DTIProcess)

category: Diffusion.Diffusion Weighted Images

description: dtiestim is a tool that takes in a set of DWIs (with --dwi_image option) in nrrd format and estimates a tensor field out of it. The output tensor file name is specified with the --tensor_output option
There are several methods to estimate the tensors which you can specify with the option --method lls|wls|nls|ml . Here is a short description of the different methods:

lls
      Linear least squares. Standard estimation technique that recovers the tensor parameters by multiplying the log of the normalized signal intensities by the pseudo-inverse of the gradient matrix. Default option.

wls
    Weighted least squares. This method is similar to the linear least squares method except that the gradient matrix is weighted by the original lls estimate. (See Salvador, R., Pena, A., Menon, D. K., Carpenter, T. A., Pickard, J. D., and Bullmore, E. T. Formal characterization and extension of the linearized diffusion tensor model. Human Brain Mapping 24, 2 (Feb. 2005), 144-155. for more information on this method). This method is recommended for most applications. The weight for each iteration can be specified with the --weight_iterations.  It is not currently the default due to occasional matrix singularities.
nls
    Non-linear least squares. This method does not take the log of the signal and requires an optimization based on levenberg-marquadt to optimize the parameters of the signal. The lls estimate is used as an initialization. For this method the step size can be specified with the --step option.
ml
    Maximum likelihood estimation. This method is experimental and is not currently recommended. For this ml method the sigma can be specified with the option --sigma and the step size can be specified with the --step option.

You can set a threshold (--threshold) to have the tensor estimated to only a subset of voxels. All the baseline voxel value higher than the threshold define the voxels where the tensors are computed. If not specified the threshold is calculated using an OTSU threshold on the baseline image.The masked generated by the -t option or by the otsu value can be saved with the --B0_mask_output option.

dtiestim also can extract a few scalar images out of the DWI set of images:

        - the average baseline image (--B0) which is the average of all the B0s.
        - the IDWI (--idwi)which is the geometric mean of the diffusion images.

You can also load a mask if you want to compute the tensors only where the voxels are non-zero (--brain_mask) or a negative mask and the tensors will be estimated where the negative mask has zero values (--bad_region_mask)

version: 1.2.0

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/Nightly/Extensions/DTIProcess

license: Copyright (c)  Casey Goodlett. All rights reserved.
  See http://www.ia.unc.edu/dev/Copyright.htm for details.
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

contributor: Casey Goodlett, Francois Budin

acknowledgements: Hans Johnson(1,3,4); Kent Williams(1); (1=University of Iowa Department of Psychiatry, 3=University of Iowa Department of Biomedical Engineering, 4=University of Iowa Department of Electrical and Computer Engineering) provided conversions to make DTIProcess compatible with Slicer execution, and simplified the stand-alone build requirements by removing the dependancies on boost and a fortran compiler.

"""

    input_spec = dtiestimInputSpec
    output_spec = dtiestimOutputSpec
    _cmd = " dtiestim "
    _outputs_filenames = {
        'B0': 'B0.nii',
        'idwi': 'idwi.nii',
        'tensor_output': 'tensor_output.nii',
        'B0_mask_output': 'B0_mask_output.nii'
    }
    _redirect_x = False


class dtiprocessInputSpec(CommandLineInputSpec):
    dti_image = File(
        desc="DTI tensor volume", exists=True, argstr="--dti_image %s")
    fa_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Fractional Anisotropy output file",
        argstr="--fa_output %s")
    md_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Mean Diffusivity output file",
        argstr="--md_output %s")
    sigma = traits.Float(desc="Scale of gradients", argstr="--sigma %f")
    fa_gradient_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Fractional Anisotropy Gradient output file",
        argstr="--fa_gradient_output %s")
    fa_gradmag_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Fractional Anisotropy Gradient Magnitude output file",
        argstr="--fa_gradmag_output %s")
    color_fa_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Color Fractional Anisotropy output file",
        argstr="--color_fa_output %s")
    principal_eigenvector_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Principal Eigenvectors Output",
        argstr="--principal_eigenvector_output %s")
    negative_eigenvector_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Negative Eigenvectors Output: create a binary image where if any of the eigen value is below zero, the voxel is set to 1, otherwise 0.",
        argstr="--negative_eigenvector_output %s")
    frobenius_norm_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Frobenius Norm Output",
        argstr="--frobenius_norm_output %s")
    lambda1_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Axial Diffusivity - Lambda 1 (largest eigenvalue) output",
        argstr="--lambda1_output %s")
    lambda2_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Lambda 2 (middle eigenvalue) output",
        argstr="--lambda2_output %s")
    lambda3_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Lambda 3 (smallest eigenvalue) output",
        argstr="--lambda3_output %s")
    RD_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="RD (Radial Diffusivity 1/2*(lambda2+lambda3)) output",
        argstr="--RD_output %s")
    rot_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Rotated tensor output file.  Must also specify the dof file.",
        argstr="--rot_output %s")
    affineitk_file = File(
        desc="Transformation file for affine transformation.  ITK format.",
        exists=True,
        argstr="--affineitk_file %s")
    dof_file = File(
        desc=
        "Transformation file for affine transformation.  This can be ITK format (or the outdated RView).",
        exists=True,
        argstr="--dof_file %s")
    newdof_file = File(
        desc=
        "Transformation file for affine transformation.  RView NEW format. (txt file output of dof2mat)",
        exists=True,
        argstr="--newdof_file %s")
    mask = File(
        desc=
        "Mask tensors. Specify --outmask if you want to save the masked tensor field, otherwise the mask is applied just for the current processing ",
        exists=True,
        argstr="--mask %s")
    outmask = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Name of the masked tensor field.",
        argstr="--outmask %s")
    hField = traits.Bool(
        desc=
        "forward and inverse transformations are h-fields instead of displacement fields",
        argstr="--hField ")
    forward = File(
        desc=
        "Forward transformation.  Assumed to be a deformation field in world coordinates, unless the --h-field option is specified.",
        exists=True,
        argstr="--forward %s")
    deformation_output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Warped tensor field based on a deformation field.  This option requires the --forward,-F transformation to be specified.",
        argstr="--deformation_output %s")
    interpolation = traits.Enum(
        "nearestneighbor",
        "linear",
        "cubic",
        desc="Interpolation type (nearestneighbor, linear, cubic)",
        argstr="--interpolation %s")
    reorientation = traits.Enum(
        "fs",
        "ppd",
        desc="Reorientation type (fs, ppd)",
        argstr="--reorientation %s")
    correction = traits.Enum(
        "none",
        "zero",
        "abs",
        "nearest",
        desc=
        "Correct the tensors if computed tensor is not semi-definite positive",
        argstr="--correction %s")
    scalar_float = traits.Bool(
        desc=
        "Write scalar [FA,MD] as unscaled float (with their actual values, otherwise scaled by 10 000).  Also causes FA to be unscaled [0..1].",
        argstr="--scalar_float ")
    DTI_double = traits.Bool(
        desc=
        "Tensor components are saved as doubles (cannot be visualized in Slicer)",
        argstr="--DTI_double ")
    verbose = traits.Bool(desc="produce verbose output", argstr="--verbose ")


class dtiprocessOutputSpec(TraitedSpec):
    fa_output = File(desc="Fractional Anisotropy output file", exists=True)
    md_output = File(desc="Mean Diffusivity output file", exists=True)
    fa_gradient_output = File(
        desc="Fractional Anisotropy Gradient output file", exists=True)
    fa_gradmag_output = File(
        desc="Fractional Anisotropy Gradient Magnitude output file",
        exists=True)
    color_fa_output = File(
        desc="Color Fractional Anisotropy output file", exists=True)
    principal_eigenvector_output = File(
        desc="Principal Eigenvectors Output", exists=True)
    negative_eigenvector_output = File(
        desc=
        "Negative Eigenvectors Output: create a binary image where if any of the eigen value is below zero, the voxel is set to 1, otherwise 0.",
        exists=True)
    frobenius_norm_output = File(desc="Frobenius Norm Output", exists=True)
    lambda1_output = File(
        desc="Axial Diffusivity - Lambda 1 (largest eigenvalue) output",
        exists=True)
    lambda2_output = File(
        desc="Lambda 2 (middle eigenvalue) output", exists=True)
    lambda3_output = File(
        desc="Lambda 3 (smallest eigenvalue) output", exists=True)
    RD_output = File(
        desc="RD (Radial Diffusivity 1/2*(lambda2+lambda3)) output",
        exists=True)
    rot_output = File(
        desc="Rotated tensor output file.  Must also specify the dof file.",
        exists=True)
    outmask = File(desc="Name of the masked tensor field.", exists=True)
    deformation_output = File(
        desc=
        "Warped tensor field based on a deformation field.  This option requires the --forward,-F transformation to be specified.",
        exists=True)


class dtiprocess(SEMLikeCommandLine):
    """title: DTIProcess (DTIProcess)

category: Diffusion.Diffusion Tensor Images

description: dtiprocess is a tool that handles tensor fields. It takes as an input a tensor field in nrrd format.
It can generate diffusion scalar properties out of the tensor field such as : FA (--fa_output), Gradient FA image (--fa_gradient_output), color FA (--color_fa_output), MD (--md_output), Frobenius norm (--frobenius_norm_output), lbd1, lbd2, lbd3 (--lambda{1,2,3}_output), binary map of voxel where if any of the eigenvalue is negative, the voxel is set to 1 (--negative_eigenvector_output)

It also creates 4D images out of the tensor field such as: Highest eigenvector map (highest eigenvector at each voxel) (--principal_eigenvector_output)

Masking capabilities: For any of the processing done with dtiprocess, it's possible to apply it on a masked region of the tensor field. You need to use the --mask option for any of the option to be applied on that tensor field sub-region only. If you want to save the masked tensor field use the option --outmask and specify the new masked tensor field file name.
dtiprocess also allows a range of transformations on the tensor fields. The transformed tensor field file name is specified with the option --deformation_output. There are 3 resampling interpolation methods specified with the tag --interpolation followed by the type to use (nearestneighbor, linear, cubic) Then you have several transformations possible to apply:

        - Affine transformations using as an input
        - itk affine transformation file (based on the itkAffineTransform class)
        - Affine transformations using rview (details and download at http://www.doc.ic.ac.uk/~dr/software/). There are 2 versions of rview both creating transformation files called dof files. The old version of rview outputs text files containing the transformation parameters. It can be read in with the --dof_file option. The new version outputs binary dof files. These dof files can be transformed into human readable file with the dof2mat tool which is part of the rview package. So you need to save the output of dof2mat into a text file which can then be used with the -- newdof_file option. Usage example: dof2mat mynewdoffile.dof >> mynewdoffile.txt       dtiprocess --dti_image mytensorfield.nhdr --newdof_file mynewdoffile.txt --rot_output myaffinetensorfield.nhdr

Non linear transformations as an input: The default transformation file type is d-field (displacement field) in nrrd format. The option to use is --forward with the name of the file. If the transformation file is a h-field you have to add the option --hField.

version: 1.0.1

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/Nightly/Extensions/DTIProcess

license: Copyright (c)  Casey Goodlett. All rights reserved.
  See http://www.ia.unc.edu/dev/Copyright.htm for details.
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

contributor: Casey Goodlett

"""

    input_spec = dtiprocessInputSpec
    output_spec = dtiprocessOutputSpec
    _cmd = " dtiprocess "
    _outputs_filenames = {
        'fa_gradmag_output': 'fa_gradmag_output.nii',
        'fa_gradient_output': 'fa_gradient_output.nii',
        'lambda1_output': 'lambda1_output.nii',
        'lambda2_output': 'lambda2_output.nii',
        'color_fa_output': 'color_fa_output.nii',
        'fa_output': 'fa_output.nii',
        'frobenius_norm_output': 'frobenius_norm_output.nii',
        'principal_eigenvector_output': 'principal_eigenvector_output.nii',
        'outmask': 'outmask.nii',
        'lambda3_output': 'lambda3_output.nii',
        'negative_eigenvector_output': 'negative_eigenvector_output.nii',
        'md_output': 'md_output.nii',
        'RD_output': 'RD_output.nii',
        'deformation_output': 'deformation_output.nii',
        'rot_output': 'rot_output.nii'
    }
    _redirect_x = False


class DWIConvertInputSpec(CommandLineInputSpec):
    conversionMode = traits.Enum(
        "DicomToNrrd",
        "DicomToFSL",
        "NrrdToFSL",
        "FSLToNrrd",
        desc=
        "Determine which conversion to perform. DicomToNrrd (default): Convert DICOM series to NRRD DicomToFSL: Convert DICOM series to NIfTI File + gradient/bvalue text files NrrdToFSL: Convert DWI NRRD file to NIfTI File + gradient/bvalue text files FSLToNrrd: Convert NIfTI File + gradient/bvalue text files to NRRD file.",
        argstr="--conversionMode %s")
    inputVolume = File(
        desc="Input DWI volume -- not used for DicomToNrrd mode.",
        exists=True,
        argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output filename (.nhdr or .nrrd)",
        argstr="--outputVolume %s")
    inputDicomDirectory = Directory(
        desc="Directory holding Dicom series",
        exists=True,
        argstr="--inputDicomDirectory %s")
    fslNIFTIFile = File(
        desc="4D NIfTI file containing gradient volumes",
        exists=True,
        argstr="--fslNIFTIFile %s")
    inputBValues = File(
        desc="The B Values are stored in FSL .bval text file format",
        exists=True,
        argstr="--inputBValues %s")
    inputBVectors = File(
        desc="The Gradient Vectors are stored in FSL .bvec text file format",
        exists=True,
        argstr="--inputBVectors %s")
    outputBValues = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "The B Values are stored in FSL .bval text file format (defaults to <outputVolume>.bval)",
        argstr="--outputBValues %s")
    outputBVectors = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "The Gradient Vectors are stored in FSL .bvec text file format (defaults to <outputVolume>.bvec)",
        argstr="--outputBVectors %s")
    fMRI = traits.Bool(
        desc="Output a NRRD file, but without gradients", argstr="--fMRI ")
    writeProtocolGradientsFile = traits.Bool(
        desc=
        "Write the protocol gradients to a file suffixed by \'.txt\' as they were specified in the procol by multiplying each diffusion gradient direction by the measurement frame.  This file is for debugging purposes only, the format is not fixed, and will likely change as debugging of new dicom formats is necessary.",
        argstr="--writeProtocolGradientsFile ")
    useIdentityMeaseurementFrame = traits.Bool(
        desc=
        "Adjust all the gradients so that the measurement frame is an identity matrix.",
        argstr="--useIdentityMeaseurementFrame ")
    useBMatrixGradientDirections = traits.Bool(
        desc=
        "Fill the nhdr header with the gradient directions and bvalues computed out of the BMatrix. Only changes behavior for Siemens data.  In some cases the standard public gradients are not properly computed.  The gradients can emperically computed from the private BMatrix fields.  In some cases the private BMatrix is consistent with the public grandients, but not in all cases, when it exists BMatrix is usually most robust.",
        argstr="--useBMatrixGradientDirections ")
    outputDirectory = traits.Either(
        traits.Bool,
        Directory(),
        hash_files=False,
        desc="Directory holding the output NRRD file",
        argstr="--outputDirectory %s")
    gradientVectorFile = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Text file giving gradient vectors",
        argstr="--gradientVectorFile %s")
    smallGradientThreshold = traits.Float(
        desc=
        "If a gradient magnitude is greater than 0 and less than smallGradientThreshold, then DWIConvert will display an error message and quit, unless the useBMatrixGradientDirections option is set.",
        argstr="--smallGradientThreshold %f")
    allowLossyConversion = traits.Bool(
        desc=
        "The only supported output type is \'short\'. Conversion from images of a different type may cause data loss due to rounding or truncation. Use with caution!",
        argstr="--allowLossyConversion ")
    transposeInputBVectors = traits.Bool(
        desc=
        "FSL input BVectors are expected to be encoded in the input file as one vector per line. If it is not the case, use this option to transpose the file as it is read.",
        argstr="--transposeInputBVectors ")


class DWIConvertOutputSpec(TraitedSpec):
    outputVolume = File(desc="Output filename (.nhdr or .nrrd)", exists=True)
    outputBValues = File(
        desc=
        "The B Values are stored in FSL .bval text file format (defaults to <outputVolume>.bval)",
        exists=True)
    outputBVectors = File(
        desc=
        "The Gradient Vectors are stored in FSL .bvec text file format (defaults to <outputVolume>.bvec)",
        exists=True)
    outputDirectory = Directory(
        desc="Directory holding the output NRRD file", exists=True)
    gradientVectorFile = File(
        desc="Text file giving gradient vectors", exists=True)


class DWIConvert(SEMLikeCommandLine):
    """title: DWIConverter

category: Diffusion.Diffusion Data Conversion

description: Converts diffusion weighted MR images in dicom series into Nrrd format for analysis in Slicer. This program has been tested on only a limited subset of DTI dicom formats available from Siemens, GE, and Phillips scanners. Work in progress to support dicom multi-frame data. The program parses dicom header to extract necessary information about measurement frame, diffusion weighting directions, b-values, etc, and write out a nrrd image. For non-diffusion weighted dicom images, it loads in an entire dicom series and writes out a single dicom volume in a .nhdr/.raw pair.

version: Version 1.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DWIConverter

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: Vince Magnotta (UIowa), Hans Johnson (UIowa), Joy Matsui (UIowa), Kent Williams (UIowa), Mark Scully (Uiowa), Xiaodong Tao (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.  Additional support for DTI data produced on Philips scanners was contributed by Vincent Magnotta and Hans Johnson at the University of Iowa.

"""

    input_spec = DWIConvertInputSpec
    output_spec = DWIConvertOutputSpec
    _cmd = " DWIConvert "
    _outputs_filenames = {
        'outputVolume': 'outputVolume.nii',
        'outputDirectory': 'outputDirectory',
        'outputBValues': 'outputBValues.bval',
        'gradientVectorFile': 'gradientVectorFile',
        'outputBVectors': 'outputBVectors.bvec'
    }
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..misc import FuzzyOverlap


def test_FuzzyOverlap_inputs():
    input_map = dict(
        in_mask=dict(extensions=None, ),
        in_ref=dict(mandatory=True, ),
        in_tst=dict(mandatory=True, ),
        out_file=dict(
            extensions=None,
            usedefault=True,
        ),
        weighting=dict(usedefault=True, ),
    )
    inputs = FuzzyOverlap.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FuzzyOverlap_outputs():
    output_map = dict(
        class_fdi=dict(),
        class_fji=dict(),
        dice=dict(),
        jaccard=dict(),
    )
    outputs = FuzzyOverlap.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import CALabel


def test_CALabel_inputs():
    input_map = dict(
        align=dict(argstr='-align', ),
        args=dict(argstr='%s', ),
        aseg=dict(
            argstr='-aseg %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-4,
        ),
        in_vol=dict(
            argstr='-r %s',
            extensions=None,
        ),
        intensities=dict(
            argstr='-r %s',
            extensions=None,
        ),
        label=dict(
            argstr='-l %s',
            extensions=None,
        ),
        no_big_ventricles=dict(argstr='-nobigventricles', ),
        num_threads=dict(),
        out_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        prior=dict(argstr='-prior %.1f', ),
        relabel_unlikely=dict(argstr='-relabel_unlikely %d %.1f', ),
        subjects_dir=dict(),
        template=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        transform=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
    )
    inputs = CALabel.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CALabel_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = CALabel.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



"""Parallel workflow execution via PBS/Torque
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open

import os
import sys

from ...interfaces.base import CommandLine
from .sgegraph import SGEGraphPlugin
from .base import logger


class PBSGraphPlugin(SGEGraphPlugin):
    """Execute using PBS/Torque

    The plugin_args input to run can be used to control the SGE execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - qsub_args : arguments to be prepended to the job execution script in the
                  qsub call

    """
    _template = """
#PBS -V
"""

    def _submit_graph(self, pyfiles, dependencies, nodes):
        batch_dir, _ = os.path.split(pyfiles[0])
        submitjobsfile = os.path.join(batch_dir, 'submit_jobs.sh')
        with open(submitjobsfile, 'wt') as fp:
            fp.writelines('#!/usr/bin/env sh\n')
            for idx, pyscript in enumerate(pyfiles):
                node = nodes[idx]
                template, qsub_args = self._get_args(node,
                                                     ["template", "qsub_args"])

                batch_dir, name = os.path.split(pyscript)
                name = '.'.join(name.split('.')[:-1])
                batchscript = '\n'.join((template, '%s %s' % (sys.executable,
                                                              pyscript)))
                batchscriptfile = os.path.join(batch_dir,
                                               'batchscript_%s.sh' % name)
                with open(batchscriptfile, 'wt') as batchfp:
                    batchfp.writelines(batchscript)
                    batchfp.close()
                deps = ''
                if idx in dependencies:
                    values = [
                        '$job%05d' % jobid for jobid in dependencies[idx]
                    ]
                    if len(values):
                        deps = '-W depend=afterok:%s' % ':'.join(values)
                fp.writelines('job%05d=`qsub %s %s %s`\n' %
                              (idx, deps, qsub_args, batchscriptfile))
        cmd = CommandLine(
            'sh',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        cmd.inputs.args = '%s' % submitjobsfile
        cmd.run()
        logger.info('submitted all jobs to queue')



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import MultiResolutionAffineRegistration


def test_MultiResolutionAffineRegistration_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedImage=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        fixedImageMask=dict(
            argstr='--fixedImageMask %s',
            extensions=None,
        ),
        fixedImageROI=dict(argstr='--fixedImageROI %s', ),
        metricTolerance=dict(argstr='--metricTolerance %f', ),
        movingImage=dict(
            argstr='%s',
            extensions=None,
            position=-1,
        ),
        numIterations=dict(argstr='--numIterations %d', ),
        numLineIterations=dict(argstr='--numLineIterations %d', ),
        resampledImage=dict(
            argstr='--resampledImage %s',
            hash_files=False,
        ),
        saveTransform=dict(
            argstr='--saveTransform %s',
            hash_files=False,
        ),
        stepSize=dict(argstr='--stepSize %f', ),
        stepTolerance=dict(argstr='--stepTolerance %f', ),
    )
    inputs = MultiResolutionAffineRegistration.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MultiResolutionAffineRegistration_outputs():
    output_map = dict(
        resampledImage=dict(extensions=None, ),
        saveTransform=dict(extensions=None, ),
    )
    outputs = MultiResolutionAffineRegistration.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..petstandarduptakevaluecomputation import PETStandardUptakeValueComputation


def test_PETStandardUptakeValueComputation_inputs():
    input_map = dict(
        OutputLabel=dict(argstr='--OutputLabel %s', ),
        OutputLabelValue=dict(argstr='--OutputLabelValue %s', ),
        SUVMax=dict(argstr='--SUVMax %s', ),
        SUVMean=dict(argstr='--SUVMean %s', ),
        SUVMin=dict(argstr='--SUVMin %s', ),
        args=dict(argstr='%s', ),
        color=dict(
            argstr='--color %s',
            extensions=None,
        ),
        csvFile=dict(
            argstr='--csvFile %s',
            hash_files=False,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        labelMap=dict(
            argstr='--labelMap %s',
            extensions=None,
        ),
        petDICOMPath=dict(argstr='--petDICOMPath %s', ),
        petVolume=dict(
            argstr='--petVolume %s',
            extensions=None,
        ),
    )
    inputs = PETStandardUptakeValueComputation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_PETStandardUptakeValueComputation_outputs():
    output_map = dict(csvFile=dict(extensions=None, ), )
    outputs = PETStandardUptakeValueComputation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Smooth


def test_Smooth_inputs():
    input_map = dict(
        data_type=dict(field='dtype', ),
        fwhm=dict(field='fwhm', ),
        implicit_masking=dict(field='im', ),
        in_files=dict(
            copyfile=False,
            field='data',
            mandatory=True,
        ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        out_prefix=dict(
            field='prefix',
            usedefault=True,
        ),
        paths=dict(),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
    )
    inputs = Smooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Smooth_outputs():
    output_map = dict(smoothed_files=dict(), )
    outputs = Smooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function, unicode_literals

from nipype.interfaces import utility


def test_csvReader(tmpdir):
    header = "files,labels,erosion\n"
    lines = ["foo,hello,300.1\n", "bar,world,5\n", "baz,goodbye,0.3\n"]
    for x in range(2):
        name = tmpdir.join("testfile.csv").strpath
        with open(name, 'w') as fid:
            reader = utility.CSVReader()
            if x % 2 == 0:
                fid.write(header)
                reader.inputs.header = True
            fid.writelines(lines)
            fid.flush()
            reader.inputs.in_file = name
            out = reader.run()
            if x % 2 == 0:
                assert out.outputs.files == ['foo', 'bar', 'baz']
                assert out.outputs.labels == ['hello', 'world', 'goodbye']
                assert out.outputs.erosion == ['300.1', '5', '0.3']
            else:
                assert out.outputs.column_0 == ['foo', 'bar', 'baz']
                assert out.outputs.column_1 == ['hello', 'world', 'goodbye']
                assert out.outputs.column_2 == ['300.1', '5', '0.3']



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import MedicAlgorithmThresholdToBinaryMask


def test_MedicAlgorithmThresholdToBinaryMask_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inLabel=dict(
            argstr='--inLabel %s',
            sep=';',
        ),
        inMaximum=dict(argstr='--inMaximum %f', ),
        inMinimum=dict(argstr='--inMinimum %f', ),
        inUse=dict(argstr='--inUse %s', ),
        null=dict(argstr='--null %s', ),
        outBinary=dict(
            argstr='--outBinary %s',
            sep=';',
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = MedicAlgorithmThresholdToBinaryMask.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MedicAlgorithmThresholdToBinaryMask_outputs():
    output_map = dict()
    outputs = MedicAlgorithmThresholdToBinaryMask.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Managing statistical maps
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
import os
import nibabel as nb
import numpy as np

from ..interfaces.base import (
    BaseInterfaceInputSpec, TraitedSpec, SimpleInterface,
    traits, InputMultiPath, File
)
from ..utils.filemanip import split_filename


class ActivationCountInputSpec(BaseInterfaceInputSpec):
    in_files = InputMultiPath(File(exists=True), mandatory=True,
                              desc='input file, generally a list of z-stat maps')
    threshold = traits.Float(
        mandatory=True, desc='binarization threshold. E.g. a threshold of 1.65 '
                             'corresponds to a two-sided Z-test of p<.10')


class ActivationCountOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='output activation count map')
    acm_pos = File(exists=True, desc='positive activation count map')
    acm_neg = File(exists=True, desc='negative activation count map')


class ActivationCount(SimpleInterface):
    """
    Calculate a simple Activation Count Maps

    Adapted from: https://github.com/poldracklab/CNP_task_analysis/\
    blob/61c27f5992db9d8800884f8ffceb73e6957db8af/CNP_2nd_level_ACM.py
    """
    input_spec = ActivationCountInputSpec
    output_spec = ActivationCountOutputSpec

    def _run_interface(self, runtime):
        allmaps = nb.concat_images(self.inputs.in_files).get_data()
        acm_pos = np.mean(allmaps > self.inputs.threshold,
                          axis=3, dtype=np.float32)
        acm_neg = np.mean(allmaps < -1.0 * self.inputs.threshold,
                          axis=3, dtype=np.float32)
        acm_diff = acm_pos - acm_neg

        template_fname = self.inputs.in_files[0]
        ext = split_filename(template_fname)[2]
        fname_fmt = os.path.join(runtime.cwd, 'acm_{}' + ext).format

        self._results['out_file'] = fname_fmt('diff')
        self._results['acm_pos'] = fname_fmt('pos')
        self._results['acm_neg'] = fname_fmt('neg')

        img = nb.load(template_fname)
        img.__class__(acm_diff, img.affine, img.header).to_filename(
            self._results['out_file'])
        img.__class__(acm_pos, img.affine, img.header).to_filename(
            self._results['acm_pos'])
        img.__class__(acm_neg, img.affine, img.header).to_filename(
            self._results['acm_neg'])

        return runtime



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Blur


def test_Blur_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        clobber=dict(
            argstr='-clobber',
            usedefault=True,
        ),
        dimensions=dict(argstr='-dimensions %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fwhm=dict(
            argstr='-fwhm %s',
            mandatory=True,
            xor=('fwhm', 'fwhm3d', 'standard_dev'),
        ),
        fwhm3d=dict(
            argstr='-3dfwhm %s %s %s',
            mandatory=True,
            xor=('fwhm', 'fwhm3d', 'standard_dev'),
        ),
        gaussian=dict(
            argstr='-gaussian',
            xor=('gaussian', 'rect'),
        ),
        gradient=dict(argstr='-gradient', ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        no_apodize=dict(argstr='-no_apodize', ),
        output_file_base=dict(
            argstr='%s',
            extensions=None,
            position=-1,
        ),
        partial=dict(argstr='-partial', ),
        rect=dict(
            argstr='-rect',
            xor=('gaussian', 'rect'),
        ),
        standard_dev=dict(
            argstr='-standarddev %s',
            mandatory=True,
            xor=('fwhm', 'fwhm3d', 'standard_dev'),
        ),
    )
    inputs = Blur.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Blur_outputs():
    output_map = dict(
        gradient_dxyz=dict(extensions=None, ),
        output_file=dict(extensions=None, ),
        partial_dx=dict(extensions=None, ),
        partial_dxyz=dict(extensions=None, ),
        partial_dy=dict(extensions=None, ),
        partial_dz=dict(extensions=None, ),
    )
    outputs = Blur.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Extend numpy's decorators to use nipype's gui and data labels.
"""
from numpy.testing import dec

from nibabel.data import DataError


def make_label_dec(label, ds=None):
    """Factory function to create a decorator that applies one or more labels.

    Parameters
    ----------
    label : str or sequence
        One or more labels that will be applied by the decorator to the
        functions it decorates. Labels are attributes of the decorated function
        with their value set to True.
    ds : str
        An optional docstring for the resulting decorator. If not given, a
        default docstring is auto-generated.

    Returns
    -------
    ldec : function
        A decorator.

    Examples
    --------
    >>> slow = make_label_dec('slow')
    >>> slow.__doc__
    "Labels a test as 'slow'"

    >>> rare = make_label_dec(['slow','hard'],
    ... "Mix labels 'slow' and 'hard' for rare tests")
    >>> @rare
    ... def f(): pass
    ...
    >>>
    >>> f.slow
    True
    >>> f.hard
    True
    """
    if isinstance(label, str):
        labels = [label]
    else:
        labels = label
    # Validate that the given label(s) are OK for use in setattr() by doing a
    # dry run on a dummy function.
    tmp = lambda: None
    for label in labels:
        setattr(tmp, label, True)
    # This is the actual decorator we'll return

    def decor(f):
        for label in labels:
            setattr(f, label, True)
        return f

    # Apply the user's docstring
    if ds is None:
        ds = "Labels a test as %r" % label
        decor.__doc__ = ds
    return decor


# For tests that need further review


def needs_review(msg):
    """ Skip a test that needs further review.

    Parameters
    ----------
    msg : string
        msg regarding the review that needs to be done
    """

    def skip_func(func):
        return dec.skipif(True, msg)(func)

    return skip_func


# Easier version of the numpy knownfailure
def knownfailure(f):
    return dec.knownfailureif(True)(f)


def if_datasource(ds, msg):
    try:
        ds.get_filename()
    except DataError:
        return dec.skipif(True, msg)
    return lambda f: f



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The workbench module provides classes for interfacing with `connectome workbench
<https://www.humanconnectome.org/software/connectome-workbench>`_ tools.

`Connectome Workbench is an open source, freely available visualization and
 discovery tool used to map neuroimaging data, especially data generated by the
 Human Connectome Project.
"""

from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
import os
import re

from ... import logging
from ...utils.filemanip import split_filename
from ..base import CommandLine, PackageInfo

iflogger = logging.getLogger('nipype.interface')


class Info(PackageInfo):
    """
    Handle `wb_command` version information.
    """

    version_cmd = 'wb_command -version'

    @staticmethod
    def parse_version(raw_info):
        m = re.search(r'\nVersion (\S+)', raw_info)
        return m.groups()[0] if m else None


class WBCommand(CommandLine):
    """Base support for workbench commands."""

    @property
    def version(self):
        return Info.version()

    def _gen_filename(self, name, outdir=None, suffix='', ext=None):
        """Generate a filename based on the given parameters.
        The filename will take the form: <basename><suffix><ext>.
        Parameters
        ----------
        name : str
            Filename to base the new filename on.
        suffix : str
            Suffix to add to the `basename`.  (defaults is '' )
        ext : str
            Extension to use for the new filename.
        Returns
        -------
        fname : str
            New filename based on given parameters.
        """
        if not name:
            raise ValueError("Cannot generate filename - filename not set")

        _, fname, fext = split_filename(name)
        if ext is None:
            ext = fext
        if outdir is None:
            outdir = '.'
        return os.path.join(outdir, fname + suffix + ext)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import ANTS


def test_ANTS_inputs():
    input_map = dict(
        affine_gradient_descent_option=dict(argstr='%s', ),
        args=dict(argstr='%s', ),
        delta_time=dict(requires=['number_of_time_steps'], ),
        dimension=dict(
            argstr='%d',
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_image=dict(mandatory=True, ),
        gradient_step_length=dict(requires=['transformation_model'], ),
        metric=dict(mandatory=True, ),
        metric_weight=dict(
            mandatory=True,
            requires=['metric'],
            usedefault=True,
        ),
        mi_option=dict(
            argstr='--MI-option %s',
            sep='x',
        ),
        moving_image=dict(
            argstr='%s',
            mandatory=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        number_of_affine_iterations=dict(
            argstr='--number-of-affine-iterations %s',
            sep='x',
        ),
        number_of_iterations=dict(
            argstr='--number-of-iterations %s',
            sep='x',
        ),
        number_of_time_steps=dict(requires=['gradient_step_length'], ),
        output_transform_prefix=dict(
            argstr='--output-naming %s',
            mandatory=True,
            usedefault=True,
        ),
        radius=dict(
            mandatory=True,
            requires=['metric'],
        ),
        regularization=dict(argstr='%s', ),
        regularization_deformation_field_sigma=dict(
            requires=['regularization'], ),
        regularization_gradient_field_sigma=dict(requires=['regularization'
                                                           ], ),
        smoothing_sigmas=dict(
            argstr='--gaussian-smoothing-sigmas %s',
            sep='x',
        ),
        subsampling_factors=dict(
            argstr='--subsampling-factors %s',
            sep='x',
        ),
        symmetry_type=dict(requires=['delta_time'], ),
        transformation_model=dict(
            argstr='%s',
            mandatory=True,
        ),
        use_histogram_matching=dict(
            argstr='%s',
            usedefault=True,
        ),
    )
    inputs = ANTS.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ANTS_outputs():
    output_map = dict(
        affine_transform=dict(extensions=None, ),
        inverse_warp_transform=dict(extensions=None, ),
        metaheader=dict(extensions=None, ),
        metaheader_raw=dict(extensions=None, ),
        warp_transform=dict(extensions=None, ),
    )
    outputs = ANTS.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import GLM


def test_GLM_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        contrasts=dict(
            argstr='-c %s',
            extensions=None,
        ),
        dat_norm=dict(argstr='--dat_norm', ),
        demean=dict(argstr='--demean', ),
        des_norm=dict(argstr='--des_norm', ),
        design=dict(
            argstr='-d %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        dof=dict(argstr='--dof=%d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        out_cope=dict(
            argstr='--out_cope=%s',
            extensions=None,
        ),
        out_data_name=dict(
            argstr='--out_data=%s',
            extensions=None,
        ),
        out_f_name=dict(
            argstr='--out_f=%s',
            extensions=None,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            keep_extension=True,
            name_source='in_file',
            name_template='%s_glm',
            position=3,
        ),
        out_p_name=dict(
            argstr='--out_p=%s',
            extensions=None,
        ),
        out_pf_name=dict(
            argstr='--out_pf=%s',
            extensions=None,
        ),
        out_res_name=dict(
            argstr='--out_res=%s',
            extensions=None,
        ),
        out_sigsq_name=dict(
            argstr='--out_sigsq=%s',
            extensions=None,
        ),
        out_t_name=dict(
            argstr='--out_t=%s',
            extensions=None,
        ),
        out_varcb_name=dict(
            argstr='--out_varcb=%s',
            extensions=None,
        ),
        out_vnscales_name=dict(
            argstr='--out_vnscales=%s',
            extensions=None,
        ),
        out_z_name=dict(
            argstr='--out_z=%s',
            extensions=None,
        ),
        output_type=dict(),
        var_norm=dict(argstr='--vn', ),
    )
    inputs = GLM.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_GLM_outputs():
    output_map = dict(
        out_cope=dict(),
        out_data=dict(),
        out_f=dict(),
        out_file=dict(extensions=None, ),
        out_p=dict(),
        out_pf=dict(),
        out_res=dict(),
        out_sigsq=dict(),
        out_t=dict(),
        out_varcb=dict(),
        out_vnscales=dict(),
        out_z=dict(),
    )
    outputs = GLM.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import Atropos


def test_Atropos_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        convergence_threshold=dict(requires=['n_iterations'], ),
        dimension=dict(
            argstr='--image-dimensionality %d',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        icm_use_synchronous_update=dict(argstr='%s', ),
        initialization=dict(
            argstr='%s',
            mandatory=True,
            requires=['number_of_tissue_classes'],
        ),
        intensity_images=dict(
            argstr='--intensity-image %s...',
            mandatory=True,
        ),
        likelihood_model=dict(argstr='--likelihood-model %s', ),
        mask_image=dict(
            argstr='--mask-image %s',
            extensions=None,
            mandatory=True,
        ),
        maximum_number_of_icm_terations=dict(
            requires=['icm_use_synchronous_update'], ),
        mrf_radius=dict(requires=['mrf_smoothing_factor'], ),
        mrf_smoothing_factor=dict(argstr='%s', ),
        n_iterations=dict(argstr='%s', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        number_of_tissue_classes=dict(mandatory=True, ),
        out_classified_image_name=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_posteriors_name_template=dict(usedefault=True, ),
        posterior_formulation=dict(argstr='%s', ),
        prior_probability_images=dict(),
        prior_probability_threshold=dict(requires=['prior_weighting'], ),
        prior_weighting=dict(),
        save_posteriors=dict(),
        use_mixture_model_proportions=dict(requires=['posterior_formulation'
                                                     ], ),
        use_random_seed=dict(
            argstr='--use-random-seed %d',
            usedefault=True,
        ),
    )
    inputs = Atropos.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Atropos_outputs():
    output_map = dict(
        classified_image=dict(extensions=None, ),
        posteriors=dict(),
    )
    outputs = Atropos.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""The dcm2nii module provides basic functions for dicom conversion
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import str, open
import os
import re
from copy import deepcopy
import itertools as it
from glob import iglob

from ..utils.filemanip import split_filename
from .base import (CommandLine, CommandLineInputSpec, InputMultiPath, traits,
                   TraitedSpec, OutputMultiPath, isdefined, File, Directory,
                   PackageInfo)


class Info(PackageInfo):
    """Handle dcm2niix version information"""

    version_cmd = 'dcm2niix'

    @staticmethod
    def parse_version(raw_info):
        m = re.search(r'version (\S+)', raw_info)
        return m.groups()[0] if m else None


class Dcm2niiInputSpec(CommandLineInputSpec):
    source_names = InputMultiPath(
        File(exists=True),
        argstr="%s",
        position=-1,
        copyfile=False,
        mandatory=True,
        xor=['source_dir'])
    source_dir = Directory(
        exists=True,
        argstr="%s",
        position=-1,
        mandatory=True,
        xor=['source_names'])
    anonymize = traits.Bool(
        True,
        argstr='-a',
        usedefault=True,
        desc="Remove identifying information")
    config_file = File(
        exists=True,
        argstr="-b %s",
        genfile=True,
        desc="Load settings from specified inifile")
    collapse_folders = traits.Bool(
        True, argstr='-c', usedefault=True, desc="Collapse input folders")
    date_in_filename = traits.Bool(
        True, argstr='-d', usedefault=True, desc="Date in filename")
    events_in_filename = traits.Bool(
        True,
        argstr='-e',
        usedefault=True,
        desc="Events (series/acq) in filename")
    source_in_filename = traits.Bool(
        False, argstr='-f', usedefault=True, desc="Source filename")
    gzip_output = traits.Bool(
        False, argstr='-g', usedefault=True, desc="Gzip output (.gz)")
    id_in_filename = traits.Bool(
        False, argstr='-i', usedefault=True, desc="ID  in filename")
    nii_output = traits.Bool(
        True,
        argstr='-n',
        usedefault=True,
        desc="Save as .nii - if no, create .hdr/.img pair")
    output_dir = Directory(
        exists=True,
        argstr='-o %s',
        genfile=True,
        desc="Output dir - if unspecified, source directory is used")
    protocol_in_filename = traits.Bool(
        True, argstr='-p', usedefault=True, desc="Protocol in filename")
    reorient = traits.Bool(
        argstr='-r', desc="Reorient image to nearest orthogonal")
    spm_analyze = traits.Bool(
        argstr='-s', xor=['nii_output'], desc="SPM2/Analyze not SPM5/NIfTI")
    convert_all_pars = traits.Bool(
        True,
        argstr='-v',
        usedefault=True,
        desc="Convert every image in directory")
    reorient_and_crop = traits.Bool(
        False,
        argstr='-x',
        usedefault=True,
        desc="Reorient and crop 3D images")


class Dcm2niiOutputSpec(TraitedSpec):
    converted_files = OutputMultiPath(File(exists=True))
    reoriented_files = OutputMultiPath(File(exists=True))
    reoriented_and_cropped_files = OutputMultiPath(File(exists=True))
    bvecs = OutputMultiPath(File(exists=True))
    bvals = OutputMultiPath(File(exists=True))


class Dcm2nii(CommandLine):
    """Uses MRIcron's dcm2nii to convert dicom files

    Examples
    ========

    >>> from nipype.interfaces.dcm2nii import Dcm2nii
    >>> converter = Dcm2nii()
    >>> converter.inputs.source_names = ['functional_1.dcm', 'functional_2.dcm']
    >>> converter.inputs.gzip_output = True
    >>> converter.inputs.output_dir = '.'
    >>> converter.cmdline  # doctest: +ELLIPSIS
    'dcm2nii -a y -c y -b config.ini -v y -d y -e y -g y -i n -n y -o . -p y -x n -f n functional_1.dcm'
"""

    input_spec = Dcm2niiInputSpec
    output_spec = Dcm2niiOutputSpec
    _cmd = 'dcm2nii'

    def _format_arg(self, opt, spec, val):
        if opt in [
                'anonymize', 'collapse_folders', 'date_in_filename',
                'events_in_filename', 'source_in_filename', 'gzip_output',
                'id_in_filename', 'nii_output', 'protocol_in_filename',
                'reorient', 'spm_analyze', 'convert_all_pars',
                'reorient_and_crop'
        ]:
            spec = deepcopy(spec)
            if val:
                spec.argstr += ' y'
            else:
                spec.argstr += ' n'
                val = True
        if opt == 'source_names':
            return spec.argstr % val[0]
        return super(Dcm2nii, self)._format_arg(opt, spec, val)

    def _run_interface(self, runtime):
        self._config_created = False
        new_runtime = super(Dcm2nii, self)._run_interface(runtime)
        (self.output_files, self.reoriented_files,
         self.reoriented_and_cropped_files, self.bvecs,
         self.bvals) = self._parse_stdout(new_runtime.stdout)
        if self._config_created:
            os.remove('config.ini')
        return new_runtime

    def _parse_stdout(self, stdout):
        files = []
        reoriented_files = []
        reoriented_and_cropped_files = []
        bvecs = []
        bvals = []
        skip = False
        last_added_file = None
        for line in stdout.split("\n"):
            if not skip:
                out_file = None
                if line.startswith("Saving "):
                    out_file = line[len("Saving "):]
                elif line.startswith("GZip..."):
                    # for gzipped output files are not absolute
                    fname = line[len("GZip..."):]
                    if len(files) and os.path.basename(
                            files[-1]) == fname[:-3]:
                        # we are seeing a previously reported conversion
                        # as being saved in gzipped form -- remove the
                        # obsolete, uncompressed file
                        files.pop()
                    if isdefined(self.inputs.output_dir):
                        output_dir = self.inputs.output_dir
                    else:
                        output_dir = self._gen_filename('output_dir')
                    out_file = os.path.abspath(os.path.join(output_dir, fname))
                elif line.startswith("Number of diffusion directions "):
                    if last_added_file:
                        base, filename, ext = split_filename(last_added_file)
                        bvecs.append(os.path.join(base, filename + ".bvec"))
                        bvals.append(os.path.join(base, filename + ".bval"))
                elif line.startswith("Removed DWI from DTI scan"):
                    # such line can only follow the 'diffusion' case handled
                    # just above
                    for l in (bvecs, bvals):
                        l[-1] = os.path.join(
                            os.path.dirname(l[-1]),
                            'x%s' % (os.path.basename(l[-1]), ))
                elif re.search('.*->(.*)', line):
                    val = re.search('.*->(.*)', line)
                    val = val.groups()[0]
                    if isdefined(self.inputs.output_dir):
                        output_dir = self.inputs.output_dir
                    else:
                        output_dir = self._gen_filename('output_dir')
                    val = os.path.join(output_dir, val)
                    if os.path.exists(val):
                        out_file = val

                if out_file:
                    if out_file not in files:
                        files.append(out_file)
                        last_added_file = out_file
                    continue

                if line.startswith("Reorienting as "):
                    reoriented_files.append(line[len("Reorienting as "):])
                    skip = True
                    continue
                elif line.startswith("Cropping NIfTI/Analyze image "):
                    base, filename = os.path.split(
                        line[len("Cropping NIfTI/Analyze image "):])
                    filename = "c" + filename
                    if os.path.exists(os.path.join(
                            base, filename)) or self.inputs.reorient_and_crop:
                        # if reorient&crop is true but the file doesn't exist, this errors when setting outputs
                        reoriented_and_cropped_files.append(
                            os.path.join(base, filename))
                        skip = True
                        continue

            skip = False
        return files, reoriented_files, reoriented_and_cropped_files, bvecs, bvals

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['converted_files'] = self.output_files
        outputs['reoriented_files'] = self.reoriented_files
        outputs[
            'reoriented_and_cropped_files'] = self.reoriented_and_cropped_files
        outputs['bvecs'] = self.bvecs
        outputs['bvals'] = self.bvals
        return outputs

    def _gen_filename(self, name):
        if name == 'output_dir':
            return os.getcwd()
        elif name == 'config_file':
            self._config_created = True
            config_file = "config.ini"
            with open(config_file, "w") as f:
                # disable interactive mode
                f.write("[BOOL]\nManualNIfTIConv=0\n")
            return config_file
        return None


class Dcm2niixInputSpec(CommandLineInputSpec):
    source_names = InputMultiPath(
        File(exists=True),
        argstr="%s",
        position=-1,
        copyfile=False,
        mandatory=True,
        desc=('A set of filenames to be converted. Note that the current '
              'version (1.0.20180328) of dcm2niix converts any files in the '
              'directory. To only convert specific files they should be in an '
              'isolated directory'),
        xor=['source_dir'])
    source_dir = Directory(
        exists=True,
        argstr="%s",
        position=-1,
        mandatory=True,
        desc='A directory containing dicom files to be converted',
        xor=['source_names'])
    out_filename = traits.Str(
        argstr="-f %s",
        desc="Output filename template ("
             "%a=antenna (coil) number, "
             "%c=comments, "
             "%d=description, "
             "%e=echo number, "
             "%f=folder name, "
             "%i=ID of patient, "
             "%j=seriesInstanceUID, "
             "%k=studyInstanceUID, "
             "%m=manufacturer, "
             "%n=name of patient, "
             "%p=protocol, "
             "%s=series number, "
             "%t=time, "
             "%u=acquisition number, "
             "%v=vendor, "
             "%x=study ID; "
             "%z=sequence name)")
    output_dir = Directory(
        ".",
        usedefault=True,
        exists=True,
        argstr='-o %s',
        desc="Output directory")
    bids_format = traits.Bool(
        True,
        argstr='-b',
        usedefault=True,
        desc="Create a BIDS sidecar file")
    anon_bids = traits.Bool(
        argstr='-ba',
        requires=["bids_format"],
        desc="Anonymize BIDS")
    compress = traits.Enum(
        'y', 'i', 'n', '3',
        argstr='-z %s',
        usedefault=True,
        desc="Gzip compress images - [y=pigz, i=internal, n=no, 3=no,3D]")
    merge_imgs = traits.Bool(
        False,
        argstr='-m',
        usedefault=True,
        desc="merge 2D slices from same series")
    single_file = traits.Bool(
        False,
        argstr='-s',
        usedefault=True,
        desc="Single file mode")
    verbose = traits.Bool(
        False,
        argstr='-v',
        usedefault=True,
        desc="Verbose output")
    crop = traits.Bool(
        False,
        argstr='-x',
        usedefault=True,
        desc="Crop 3D T1 acquisitions")
    has_private = traits.Bool(
        False,
        argstr='-t',
        usedefault=True,
        desc="Text notes including private patient details")
    compression = traits.Enum(
        1, 2, 3, 4, 5, 6, 7, 8, 9,
        argstr='-%d',
        desc="Gz compression level (1=fastest, 9=smallest)")
    comment = traits.Str(
        argstr='-c %s',
        desc="Comment stored as NIfTI aux_file")
    ignore_deriv = traits.Bool(
        argstr='-i',
        desc="Ignore derived, localizer and 2D images")
    series_numbers = InputMultiPath(
        traits.Str(),
        argstr='-n %s...',
        desc="Selectively convert by series number - can be used up to 16 times")
    philips_float = traits.Bool(
        argstr='-p',
        desc="Philips precise float (not display) scaling")
    to_nrrd = traits.Bool(
        argstr="-e",
        desc="Export as NRRD instead of NIfTI")


class Dcm2niixOutputSpec(TraitedSpec):
    converted_files = OutputMultiPath(File(exists=True))
    bvecs = OutputMultiPath(File(exists=True))
    bvals = OutputMultiPath(File(exists=True))
    bids = OutputMultiPath(File(exists=True))


class Dcm2niix(CommandLine):
    """Uses Chris Rorden's dcm2niix to convert dicom files

    Examples
    ========

    >>> from nipype.interfaces.dcm2nii import Dcm2niix
    >>> converter = Dcm2niix()
    >>> converter.inputs.source_dir = 'dicomdir'
    >>> converter.inputs.compression = 5
    >>> converter.inputs.output_dir = 'ds005'
    >>> converter.cmdline
    'dcm2niix -b y -z y -5 -x n -t n -m n -o ds005 -s n -v n dicomdir'
    >>> converter.run() # doctest: +SKIP

    In the example below, we note that the current version of dcm2niix
    converts any files in the directory containing the files in the list. We
    also do not support nested filenames with this option. **Thus all files
    must have a common root directory.**

    >>> converter = Dcm2niix()
    >>> converter.inputs.source_names = ['functional_1.dcm', 'functional_2.dcm']
    >>> converter.inputs.compression = 5
    >>> converter.inputs.output_dir = 'ds005'
    >>> converter.cmdline
    'dcm2niix -b y -z y -5 -x n -t n -m n -o ds005 -s n -v n .'
    >>> converter.run() # doctest: +SKIP
    """

    input_spec = Dcm2niixInputSpec
    output_spec = Dcm2niixOutputSpec
    _cmd = 'dcm2niix'

    @property
    def version(self):
        return Info.version()

    def _format_arg(self, opt, spec, val):
        bools = [
            'bids_format', 'merge_imgs', 'single_file', 'verbose', 'crop',
            'has_private', 'anon_bids', 'ignore_deriv', 'philips_float',
            'to_nrrd',
        ]
        if opt in bools:
            spec = deepcopy(spec)
            if val:
                spec.argstr += ' y'
            else:
                spec.argstr += ' n'
                val = True
        if opt == 'source_names':
            return spec.argstr % (os.path.dirname(val[0]) or '.')
        return super(Dcm2niix, self)._format_arg(opt, spec, val)

    def _run_interface(self, runtime):
        # may use return code 1 despite conversion
        runtime = super(Dcm2niix, self)._run_interface(
            runtime, correct_return_codes=(0, 1, ))
        self._parse_files(self._parse_stdout(runtime.stdout))
        return runtime

    def _parse_stdout(self, stdout):
        filenames = []
        for line in stdout.split("\n"):
            if line.startswith("Convert "):  # output
                fname = str(re.search(r'\S+/\S+', line).group(0))
                filenames.append(os.path.abspath(fname))
        return filenames

    def _parse_files(self, filenames):
        outfiles, bvals, bvecs, bids = [], [], [], []
        outtypes = [".bval", ".bvec", ".json", ".txt"]
        if self.inputs.to_nrrd:
            outtypes += [".nrrd", ".nhdr", ".raw.gz"]
        else:
            outtypes += [".nii", ".nii.gz"]

        for filename in filenames:
            # search for relevant files, and sort accordingly
            for fl in search_files(filename, outtypes):
                if (
                    fl.endswith(".nii") or
                    fl.endswith(".gz") or
                    fl.endswith(".nrrd") or
                    fl.endswith(".nhdr")
                ):
                    outfiles.append(fl)
                elif fl.endswith(".bval"):
                    bvals.append(fl)
                elif fl.endswith(".bvec"):
                    bvecs.append(fl)
                elif fl.endswith(".json") or fl.endswith(".txt"):
                    bids.append(fl)
        self.output_files = outfiles
        self.bvecs = bvecs
        self.bvals = bvals
        self.bids = bids

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['converted_files'] = self.output_files
        outputs['bvecs'] = self.bvecs
        outputs['bvals'] = self.bvals
        outputs['bids'] = self.bids
        return outputs

# https://stackoverflow.com/a/4829130
def search_files(prefix, outtypes):
    return it.chain.from_iterable(iglob(prefix + outtype) for outtype in outtypes)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..connectivity import Conmat


def test_Conmat_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-inputfile %s',
            extensions=None,
            mandatory=True,
        ),
        output_root=dict(
            argstr='-outputroot %s',
            extensions=None,
            genfile=True,
        ),
        scalar_file=dict(
            argstr='-scalarfile %s',
            extensions=None,
            requires=['tract_stat'],
        ),
        target_file=dict(
            argstr='-targetfile %s',
            extensions=None,
            mandatory=True,
        ),
        targetname_file=dict(
            argstr='-targetnamefile %s',
            extensions=None,
        ),
        tract_prop=dict(
            argstr='-tractstat %s',
            units='NA',
            xor=['tract_stat'],
        ),
        tract_stat=dict(
            argstr='-tractstat %s',
            requires=['scalar_file'],
            units='NA',
            xor=['tract_prop'],
        ),
    )
    inputs = Conmat.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Conmat_outputs():
    output_map = dict(
        conmat_sc=dict(extensions=None, ),
        conmat_ts=dict(extensions=None, ),
    )
    outputs = Conmat.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" General matlab interface code """
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open
import os

from .. import config
from .base import (CommandLineInputSpec, InputMultiPath, isdefined,
                   CommandLine, traits, File, Directory)


def get_matlab_command():
    if 'NIPYPE_NO_MATLAB' in os.environ:
        return None

    try:
        matlab_cmd = os.environ['MATLABCMD']
    except:
        matlab_cmd = 'matlab'

    try:
        res = CommandLine(
            command='which',
            args=matlab_cmd,
            resource_monitor=False,
            terminal_output='allatonce').run()
        matlab_path = res.runtime.stdout.strip()
    except Exception:
        return None
    return matlab_cmd


no_matlab = get_matlab_command() is None


class MatlabInputSpec(CommandLineInputSpec):
    """ Basic expected inputs to Matlab interface """

    script = traits.Str(
        argstr='-r \"%s;exit\"',
        desc='m-code to run',
        mandatory=True,
        position=-1)
    uses_mcr = traits.Bool(
        desc='use MCR interface',
        xor=['nodesktop', 'nosplash', 'single_comp_thread'],
        nohash=True)
    nodesktop = traits.Bool(
        True,
        argstr='-nodesktop',
        usedefault=True,
        desc='Switch off desktop mode on unix platforms',
        nohash=True)
    nosplash = traits.Bool(
        True,
        argstr='-nosplash',
        usedefault=True,
        desc='Switch of splash screen',
        nohash=True)
    logfile = File(argstr='-logfile %s', desc='Save matlab output to log')
    single_comp_thread = traits.Bool(
        argstr="-singleCompThread",
        desc="force single threaded operation",
        nohash=True)
    # non-commandline options
    mfile = traits.Bool(True, desc='Run m-code using m-file', usedefault=True)
    script_file = File(
        'pyscript.m', usedefault=True, desc='Name of file to write m-code to')
    paths = InputMultiPath(Directory(), desc='Paths to add to matlabpath')
    prescript = traits.List(
        ["ver,", "try,"],
        usedefault=True,
        desc='prescript to be added before code')
    postscript = traits.List(
        [
            "\n,catch ME,", "fprintf(2,'MATLAB code threw an exception:\\n');",
            "fprintf(2,'%s\\n',ME.message);",
            "if length(ME.stack) ~= 0, fprintf(2,'File:%s\\nName:%s\\nLine:%d\\n',ME.stack.file,ME.stack.name,ME.stack.line);, end;",
            "end;"
        ],
        desc='script added after code',
        usedefault=True)


class MatlabCommand(CommandLine):
    """Interface that runs matlab code

    >>> import nipype.interfaces.matlab as matlab
    >>> mlab = matlab.MatlabCommand(mfile=False)  # don't write script file
    >>> mlab.inputs.script = "which('who')"
    >>> out = mlab.run()  # doctest: +SKIP
    """

    _cmd = 'matlab'
    _default_matlab_cmd = None
    _default_mfile = None
    _default_paths = None
    input_spec = MatlabInputSpec

    def __init__(self, matlab_cmd=None, **inputs):
        """initializes interface to matlab
        (default 'matlab -nodesktop -nosplash')
        """
        super(MatlabCommand, self).__init__(**inputs)
        if matlab_cmd and isdefined(matlab_cmd):
            self._cmd = matlab_cmd
        elif self._default_matlab_cmd:
            self._cmd = self._default_matlab_cmd

        if self._default_mfile and not isdefined(self.inputs.mfile):
            self.inputs.mfile = self._default_mfile

        if self._default_paths and not isdefined(self.inputs.paths):
            self.inputs.paths = self._default_paths

        if not isdefined(self.inputs.single_comp_thread) and \
                not isdefined(self.inputs.uses_mcr):
            if config.getboolean('execution', 'single_thread_matlab'):
                self.inputs.single_comp_thread = True
        # For matlab commands force all output to be returned since matlab
        # does not have a clean way of notifying an error
        self.terminal_output = 'allatonce'

    @classmethod
    def set_default_matlab_cmd(cls, matlab_cmd):
        """Set the default MATLAB command line for MATLAB classes.

        This method is used to set values for all MATLAB
        subclasses.  However, setting this will not update the output
        type for any existing instances.  For these, assign the
        <instance>.inputs.matlab_cmd.
        """
        cls._default_matlab_cmd = matlab_cmd

    @classmethod
    def set_default_mfile(cls, mfile):
        """Set the default MATLAB script file format for MATLAB classes.

        This method is used to set values for all MATLAB
        subclasses.  However, setting this will not update the output
        type for any existing instances.  For these, assign the
        <instance>.inputs.mfile.
        """
        cls._default_mfile = mfile

    @classmethod
    def set_default_paths(cls, paths):
        """Set the default MATLAB paths for MATLAB classes.

        This method is used to set values for all MATLAB
        subclasses.  However, setting this will not update the output
        type for any existing instances.  For these, assign the
        <instance>.inputs.paths.
        """
        cls._default_paths = paths

    def _run_interface(self, runtime):
        self.terminal_output = 'allatonce'
        runtime = super(MatlabCommand, self)._run_interface(runtime)
        try:
            # Matlab can leave the terminal in a barbbled state
            os.system('stty sane')
        except:
            # We might be on a system where stty doesn't exist
            pass
        if 'MATLAB code threw an exception' in runtime.stderr:
            self.raise_exception(runtime)
        return runtime

    def _format_arg(self, name, trait_spec, value):
        if name in ['script']:
            argstr = trait_spec.argstr
            if self.inputs.uses_mcr:
                argstr = '%s'
            return self._gen_matlab_command(argstr, value)
        return super(MatlabCommand, self)._format_arg(name, trait_spec, value)

    def _gen_matlab_command(self, argstr, script_lines):
        """ Generates commands and, if mfile specified, writes it to disk."""
        cwd = os.getcwd()
        mfile = self.inputs.mfile or self.inputs.uses_mcr
        paths = []
        if isdefined(self.inputs.paths):
            paths = self.inputs.paths
        # prescript
        prescript = self.inputs.prescript
        postscript = self.inputs.postscript

        # prescript takes different default value depending on the mfile argument
        if mfile:
            prescript.insert(
                0,
                "fprintf(1,'Executing %s at %s:\\n',mfilename(),datestr(now));"
            )
        else:
            prescript.insert(
                0, "fprintf(1,'Executing code at %s:\\n',datestr(now));")
        for path in paths:
            prescript.append("addpath('%s');\n" % path)

        if not mfile:
            # clean up the code of comments and replace newlines with commas
            script_lines = ','.join([
                line for line in script_lines.split("\n")
                if not line.strip().startswith("%")
            ])

        script_lines = '\n'.join(prescript) + script_lines + '\n'.join(
            postscript)
        if mfile:
            with open(os.path.join(cwd, self.inputs.script_file),
                      'wt') as mfile:
                mfile.write(script_lines)
            if self.inputs.uses_mcr:
                script = '%s' % (os.path.join(cwd, self.inputs.script_file))
            else:
                script = "addpath('%s');%s" % (
                    cwd, self.inputs.script_file.split('.')[0])
        else:
            script = ''.join(script_lines.split('\n'))
        return argstr % script



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open
import os
import os.path as op
import pytest
from nipype.testing.fixtures import (create_files_in_directory_plus_dummy_file,
                                     create_surf_file_in_directory)

from nipype.pipeline import engine as pe
from nipype.interfaces import freesurfer as fs
from nipype.interfaces.base import TraitError
from nipype.interfaces.io import FreeSurferSource


@pytest.mark.skipif(fs.no_freesurfer(), reason="freesurfer is not installed")
def test_sample2surf(create_files_in_directory_plus_dummy_file):

    s2s = fs.SampleToSurface()
    # Test underlying command
    assert s2s.cmd == 'mri_vol2surf'

    # Test mandatory args exception
    with pytest.raises(ValueError):
        s2s.run()

    # Create testing files
    files, cwd = create_files_in_directory_plus_dummy_file

    # Test input settings
    s2s.inputs.source_file = files[0]
    s2s.inputs.reference_file = files[1]
    s2s.inputs.hemi = "lh"
    s2s.inputs.reg_file = files[2]
    s2s.inputs.sampling_range = .5
    s2s.inputs.sampling_units = "frac"
    s2s.inputs.sampling_method = "point"

    # Test a basic command line
    assert s2s.cmdline == (
        "mri_vol2surf "
        "--hemi lh --o %s --ref %s --reg reg.dat --projfrac 0.500 --mov %s" %
        (os.path.join(cwd, "lh.a.mgz"), files[1], files[0]))

    # Test identity
    s2sish = fs.SampleToSurface(
        source_file=files[1], reference_file=files[0], hemi="rh")
    assert s2s != s2sish

    # Test hits file name creation
    s2s.inputs.hits_file = True
    assert s2s._get_outfilename("hits_file") == os.path.join(
        cwd, "lh.a_hits.mgz")

    # Test that a 2-tuple range raises an error
    def set_illegal_range():
        s2s.inputs.sampling_range = (.2, .5)

    with pytest.raises(TraitError):
        set_illegal_range()


@pytest.mark.skipif(fs.no_freesurfer(), reason="freesurfer is not installed")
def test_surfsmooth(create_surf_file_in_directory):

    smooth = fs.SurfaceSmooth()

    # Test underlying command
    assert smooth.cmd == "mri_surf2surf"

    # Test mandatory args exception
    with pytest.raises(ValueError):
        smooth.run()

    # Create testing files
    surf, cwd = create_surf_file_in_directory

    # Test input settings
    smooth.inputs.in_file = surf
    smooth.inputs.subject_id = "fsaverage"
    fwhm = 5
    smooth.inputs.fwhm = fwhm
    smooth.inputs.hemi = "lh"

    # Test the command line
    assert smooth.cmdline == \
        ("mri_surf2surf --cortex --fwhm 5.0000 --hemi lh --sval %s --tval %s/lh.a_smooth%d.nii --s fsaverage" %
         (surf, cwd, fwhm))

    # Test identity
    shmooth = fs.SurfaceSmooth(
        subject_id="fsaverage",
        fwhm=6,
        in_file=surf,
        hemi="lh",
        out_file="lh.a_smooth.nii")
    assert smooth != shmooth


@pytest.mark.skipif(fs.no_freesurfer(), reason="freesurfer is not installed")
def test_surfxfm(create_surf_file_in_directory):

    xfm = fs.SurfaceTransform()

    # Test underlying command
    assert xfm.cmd == "mri_surf2surf"

    # Test mandatory args exception
    with pytest.raises(ValueError):
        xfm.run()

    # Create testing files
    surf, cwd = create_surf_file_in_directory

    # Test input settings
    xfm.inputs.source_file = surf
    xfm.inputs.source_subject = "my_subject"
    xfm.inputs.target_subject = "fsaverage"
    xfm.inputs.hemi = "lh"

    # Test the command line
    assert xfm.cmdline == \
        ("mri_surf2surf --hemi lh --tval %s/lh.a.fsaverage.nii --sval %s --srcsubject my_subject --trgsubject fsaverage" %
         (cwd, surf))

    # Test identity
    xfmish = fs.SurfaceTransform(
        source_subject="fsaverage",
        target_subject="my_subject",
        source_file=surf,
        hemi="lh")
    assert xfm != xfmish


@pytest.mark.skipif(fs.no_freesurfer(), reason="freesurfer is not installed")
def test_surfshots(create_files_in_directory_plus_dummy_file):

    fotos = fs.SurfaceSnapshots()

    # Test underlying command
    assert fotos.cmd == "tksurfer"

    # Test mandatory args exception
    with pytest.raises(ValueError):
        fotos.run()

    # Create testing files
    files, cwd = create_files_in_directory_plus_dummy_file

    # Test input settins
    fotos.inputs.subject_id = "fsaverage"
    fotos.inputs.hemi = "lh"
    fotos.inputs.surface = "pial"

    # Test a basic command line
    assert fotos.cmdline == "tksurfer fsaverage lh pial -tcl snapshots.tcl"

    # Test identity
    schmotos = fs.SurfaceSnapshots(
        subject_id="mysubject", hemi="rh", surface="white")
    assert fotos != schmotos

    # Test that the tcl script gets written
    fotos._write_tcl_script()
    assert os.path.exists("snapshots.tcl")

    # Test that we can use a different tcl script
    foo = open("other.tcl", "w").close()
    fotos.inputs.tcl_script = "other.tcl"
    assert fotos.cmdline == "tksurfer fsaverage lh pial -tcl other.tcl"

    # Test that the interface crashes politely if graphics aren't enabled
    try:
        hold_display = os.environ["DISPLAY"]
        del os.environ["DISPLAY"]
        with pytest.raises(RuntimeError):
            fotos.run()
        os.environ["DISPLAY"] = hold_display
    except KeyError:
        pass


@pytest.mark.skipif(fs.no_freesurfer(), reason="freesurfer is not installed")
def test_mrisexpand(tmpdir):
    fssrc = FreeSurferSource(
        subjects_dir=fs.Info.subjectsdir(), subject_id='fsaverage', hemi='lh')

    fsavginfo = fssrc.run().outputs.get()

    # dt=60 to ensure very short runtime
    expand_if = fs.MRIsExpand(
        in_file=fsavginfo['smoothwm'], out_name='expandtmp', distance=1, dt=60)

    expand_nd = pe.Node(
        fs.MRIsExpand(
            in_file=fsavginfo['smoothwm'],
            out_name='expandtmp',
            distance=1,
            dt=60),
        name='expand_node')

    # Interfaces should have same command line at instantiation
    orig_cmdline = 'mris_expand -T 60 {} 1 expandtmp'.format(
        fsavginfo['smoothwm'])
    assert expand_if.cmdline == orig_cmdline
    assert expand_nd.interface.cmdline == orig_cmdline

    # Run Node interface
    nd_res = expand_nd.run()

    # Commandlines differ
    node_cmdline = 'mris_expand -T 60 -pial {cwd}/lh.pial {cwd}/lh.smoothwm ' \
        '1 expandtmp'.format(cwd=nd_res.runtime.cwd)
    assert nd_res.runtime.cmdline == node_cmdline

    # Check output
    if_out_file = expand_if._list_outputs()['out_file']
    nd_out_file = nd_res.outputs.get()['out_file']
    # Same filename
    assert op.basename(if_out_file) == op.basename(nd_out_file)
    # Interface places output in source directory
    assert op.dirname(if_out_file) == op.dirname(fsavginfo['smoothwm'])
    # Node places output in working directory
    assert op.dirname(nd_out_file) == nd_res.runtime.cwd



# -*- coding: utf-8 -*-
from .cmtk import ROIGen, CreateMatrix, CreateNodes
from .nx import NetworkXMetrics, AverageNetworks
from .parcellation import Parcellate
from .convert import CFFConverter, MergeCNetworks
from .nbs import NetworkBasedStatistic



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The reg module provides classes for interfacing with the `niftyreg
<http://niftyreg.sourceforge.net>`_ registration command line tools.

The interfaces were written to work with niftyreg version 1.5.10
"""

from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import staticmethod
import os

from ..base import TraitedSpec, File, traits, isdefined
from .base import get_custom_path, NiftyRegCommand, NiftyRegCommandInputSpec
from ...utils.filemanip import split_filename


class RegAladinInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegAladin. """
    # Input reference file
    ref_file = File(
        exists=True,
        desc='The input reference/target image',
        argstr='-ref %s',
        mandatory=True)
    # Input floating file
    flo_file = File(
        exists=True,
        desc='The input floating/source image',
        argstr='-flo %s',
        mandatory=True)
    # No symmetric flag
    nosym_flag = traits.Bool(
        argstr='-noSym', desc='Turn off symmetric registration')
    # Rigid only registration
    rig_only_flag = traits.Bool(
        argstr='-rigOnly', desc='Do only a rigid registration')
    # Directly optimise affine flag
    desc = 'Directly optimise the affine parameters'
    aff_direct_flag = traits.Bool(argstr='-affDirect', desc=desc)
    # Input affine
    in_aff_file = File(
        exists=True,
        desc='The input affine transformation',
        argstr='-inaff %s')
    # Input reference mask
    rmask_file = File(
        exists=True, desc='The input reference mask', argstr='-rmask %s')
    # Input floating mask
    fmask_file = File(
        exists=True, desc='The input floating mask', argstr='-fmask %s')
    # Maximum number of iterations
    maxit_val = traits.Range(
        desc='Maximum number of iterations', argstr='-maxit %d', low=0)
    # Multiresolution levels
    ln_val = traits.Range(
        desc='Number of resolution levels to create', argstr='-ln %d', low=0)
    # Number of resolution levels to process
    lp_val = traits.Range(
        desc='Number of resolution levels to perform', argstr='-lp %d', low=0)
    # Smoothing to apply on reference image
    desc = 'Amount of smoothing to apply to reference image'
    smoo_r_val = traits.Float(desc=desc, argstr='-smooR %f')
    # Smoothing to apply on floating image
    desc = 'Amount of smoothing to apply to floating image'
    smoo_f_val = traits.Float(desc=desc, argstr='-smooF %f')
    # Use nifti header to initialise transformation
    desc = 'Use nifti header to initialise transformation'
    nac_flag = traits.Bool(desc=desc, argstr='-nac')
    # Use the input masks centre of mass to initialise the transformation
    desc = 'Use the masks centre of mass to initialise the transformation'
    cog_flag = traits.Bool(desc=desc, argstr='-cog')
    # Percent of blocks that are considered active.
    v_val = traits.Range(
        desc='Percent of blocks that are active', argstr='-pv %d', low=0)
    # Percent of inlier blocks
    i_val = traits.Range(
        desc='Percent of inlier blocks', argstr='-pi %d', low=0)
    # Lower threshold on reference image
    ref_low_val = traits.Float(
        desc='Lower threshold value on reference image',
        argstr='-refLowThr %f')
    # Upper threshold on reference image
    ref_up_val = traits.Float(
        desc='Upper threshold value on reference image', argstr='-refUpThr %f')
    # Lower threshold on floating image
    flo_low_val = traits.Float(
        desc='Lower threshold value on floating image', argstr='-floLowThr %f')
    # Upper threshold on floating image
    flo_up_val = traits.Float(
        desc='Upper threshold value on floating image', argstr='-floUpThr %f')
    # Platform to use
    platform_val = traits.Int(desc='Platform index', argstr='-platf %i')
    # Platform to use
    gpuid_val = traits.Int(desc='Device to use id', argstr='-gpuid %i')
    # Verbosity off
    verbosity_off_flag = traits.Bool(
        argstr='-voff', desc='Turn off verbose output')

    # Affine output transformation matrix file
    aff_file = File(
        name_source=['flo_file'],
        name_template='%s_aff.txt',
        desc='The output affine matrix file',
        argstr='-aff %s')
    # Result warped image file
    res_file = File(
        name_source=['flo_file'],
        name_template='%s_res.nii.gz',
        desc='The affine transformed floating image',
        argstr='-res %s')


class RegAladinOutputSpec(TraitedSpec):
    """ Output Spec for RegAladin. """
    aff_file = File(desc='The output affine file')
    res_file = File(desc='The output transformed image')
    desc = 'Output string in the format for reg_average'
    avg_output = traits.String(desc=desc)


class RegAladin(NiftyRegCommand):
    """Interface for executable reg_aladin from NiftyReg platform.

    Block Matching algorithm for symmetric global registration.
    Based on Modat et al., "Global image registration using
    asymmetric block-matching approach"
    J. Med. Img. 1(2) 024003, 2014, doi: 10.1117/1.JMI.1.2.024003

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegAladin()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.flo_file = 'im2.nii'
    >>> node.inputs.rmask_file = 'mask.nii'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_aladin -aff im2_aff.txt -flo im2.nii -omp 4 -ref im1.nii \
-res im2_res.nii.gz -rmask mask.nii'

    """
    _cmd = get_custom_path('reg_aladin')
    input_spec = RegAladinInputSpec
    output_spec = RegAladinOutputSpec

    def _list_outputs(self):
        outputs = super(RegAladin, self)._list_outputs()

        # Make a list of the linear transformation file and the input image
        aff = os.path.abspath(outputs['aff_file'])
        flo = os.path.abspath(self.inputs.flo_file)
        outputs['avg_output'] = '%s %s' % (aff, flo)
        return outputs


class RegF3DInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegF3D. """
    # Input reference file
    ref_file = File(
        exists=True,
        desc='The input reference/target image',
        argstr='-ref %s',
        mandatory=True)
    # Input floating file
    flo_file = File(
        exists=True,
        desc='The input floating/source image',
        argstr='-flo %s',
        mandatory=True)

    # Input Affine file
    aff_file = File(
        exists=True,
        desc='The input affine transformation file',
        argstr='-aff %s')

    # Input cpp file
    incpp_file = File(
        exists=True,
        desc='The input cpp transformation file',
        argstr='-incpp %s')

    # Reference mask
    rmask_file = File(
        exists=True, desc='Reference image mask', argstr='-rmask %s')

    # Smoothing kernel for reference
    desc = 'Smoothing kernel width for reference image'
    ref_smooth_val = traits.Float(desc=desc, argstr='-smooR %f')
    # Smoothing kernel for floating
    desc = 'Smoothing kernel width for floating image'
    flo_smooth_val = traits.Float(desc=desc, argstr='-smooF %f')

    # Lower threshold for reference image
    rlwth_thr_val = traits.Float(
        desc='Lower threshold for reference image', argstr='--rLwTh %f')
    # Upper threshold for reference image
    rupth_thr_val = traits.Float(
        desc='Upper threshold for reference image', argstr='--rUpTh %f')
    # Lower threshold for reference image
    flwth_thr_val = traits.Float(
        desc='Lower threshold for floating image', argstr='--fLwTh %f')
    # Upper threshold for reference image
    fupth_thr_val = traits.Float(
        desc='Upper threshold for floating image', argstr='--fUpTh %f')

    # Lower threshold for reference image
    desc = 'Lower threshold for reference image at the specified time point'
    rlwth2_thr_val = traits.Tuple(
        traits.Range(low=0), traits.Float, desc=desc, argstr='-rLwTh %d %f')
    # Upper threshold for reference image
    desc = 'Upper threshold for reference image at the specified time point'
    rupth2_thr_val = traits.Tuple(
        traits.Range(low=0), traits.Float, desc=desc, argstr='-rUpTh %d %f')
    # Lower threshold for reference image
    desc = 'Lower threshold for floating image at the specified time point'
    flwth2_thr_val = traits.Tuple(
        traits.Range(low=0), traits.Float, desc=desc, argstr='-fLwTh %d %f')
    # Upper threshold for reference image
    desc = 'Upper threshold for floating image at the specified time point'
    fupth2_thr_val = traits.Tuple(
        traits.Range(low=0), traits.Float, desc=desc, argstr='-fUpTh %d %f')

    # Final grid spacing along the 3 axes
    sx_val = traits.Float(
        desc='Final grid spacing along the x axes', argstr='-sx %f')
    sy_val = traits.Float(
        desc='Final grid spacing along the y axes', argstr='-sy %f')
    sz_val = traits.Float(
        desc='Final grid spacing along the z axes', argstr='-sz %f')

    # Regularisation options
    be_val = traits.Float(desc='Bending energy value', argstr='-be %f')
    le_val = traits.Float(
        desc='Linear elasticity penalty term', argstr='-le %f')
    jl_val = traits.Float(
        desc='Log of jacobian of deformation penalty value', argstr='-jl %f')
    desc = 'Do not approximate the log of jacobian penalty at control points \
only'

    no_app_jl_flag = traits.Bool(argstr='-noAppJL', desc=desc)

    # Similarity measure options
    desc = 'use NMI even when other options are specified'
    nmi_flag = traits.Bool(argstr='--nmi', desc=desc)
    desc = 'Number of bins in the histogram for reference image'
    rbn_val = traits.Range(low=0, desc=desc, argstr='--rbn %d')
    desc = 'Number of bins in the histogram for reference image'
    fbn_val = traits.Range(low=0, desc=desc, argstr='--fbn %d')
    desc = 'Number of bins in the histogram for reference image for given \
time point'

    rbn2_val = traits.Tuple(
        traits.Range(low=0),
        traits.Range(low=0),
        desc=desc,
        argstr='-rbn %d %d')

    desc = 'Number of bins in the histogram for reference image for given \
time point'

    fbn2_val = traits.Tuple(
        traits.Range(low=0),
        traits.Range(low=0),
        desc=desc,
        argstr='-fbn %d %d')

    lncc_val = traits.Float(
        desc='SD of the Gaussian for computing LNCC', argstr='--lncc %f')
    desc = 'SD of the Gaussian for computing LNCC for a given time point'
    lncc2_val = traits.Tuple(
        traits.Range(low=0), traits.Float, desc=desc, argstr='-lncc %d %f')

    ssd_flag = traits.Bool(
        desc='Use SSD as the similarity measure', argstr='--ssd')
    desc = 'Use SSD as the similarity measure for a given time point'
    ssd2_flag = traits.Range(low=0, desc=desc, argstr='-ssd %d')
    kld_flag = traits.Bool(
        desc='Use KL divergence as the similarity measure', argstr='--kld')
    desc = 'Use KL divergence as the similarity measure for a given time point'
    kld2_flag = traits.Range(low=0, desc=desc, argstr='-kld %d')
    amc_flag = traits.Bool(desc='Use additive NMI', argstr='-amc')

    nox_flag = traits.Bool(desc="Don't optimise in x direction", argstr='-nox')
    noy_flag = traits.Bool(desc="Don't optimise in y direction", argstr='-noy')
    noz_flag = traits.Bool(desc="Don't optimise in z direction", argstr='-noz')

    # Optimization options
    maxit_val = traits.Range(
        low=0,
        argstr='-maxit %d',
        desc='Maximum number of iterations per level')
    ln_val = traits.Range(
        low=0, argstr='-ln %d', desc='Number of resolution levels to create')
    lp_val = traits.Range(
        low=0, argstr='-lp %d', desc='Number of resolution levels to perform')
    nopy_flag = traits.Bool(
        desc='Do not use the multiresolution approach', argstr='-nopy')
    noconj_flag = traits.Bool(
        desc='Use simple GD optimization', argstr='-noConj')
    desc = 'Add perturbation steps after each optimization step'
    pert_val = traits.Range(low=0, desc=desc, argstr='-pert %d')

    # F3d2 options
    vel_flag = traits.Bool(
        desc='Use velocity field integration', argstr='-vel')
    fmask_file = File(
        exists=True, desc='Floating image mask', argstr='-fmask %s')

    # Other options
    desc = 'Kernel width for smoothing the metric gradient'
    smooth_grad_val = traits.Float(desc=desc, argstr='-smoothGrad %f')
    # Padding value
    pad_val = traits.Float(desc='Padding value', argstr='-pad %f')
    # verbosity off
    verbosity_off_flag = traits.Bool(
        argstr='-voff', desc='Turn off verbose output')

    # Output CPP image file
    cpp_file = File(
        name_source=['flo_file'],
        name_template='%s_cpp.nii.gz',
        desc='The output CPP file',
        argstr='-cpp %s')
    # Output warped image file
    res_file = File(
        name_source=['flo_file'],
        name_template='%s_res.nii.gz',
        desc='The output resampled image',
        argstr='-res %s')


class RegF3DOutputSpec(TraitedSpec):
    """ Output Spec for RegF3D. """
    cpp_file = File(desc='The output CPP file')
    res_file = File(desc='The output resampled image')
    invcpp_file = File(desc='The output inverse CPP file')
    invres_file = File(desc='The output inverse res file')
    desc = 'Output string in the format for reg_average'
    avg_output = traits.String(desc=desc)


class RegF3D(NiftyRegCommand):
    """Interface for executable reg_f3d from NiftyReg platform.

    Fast Free-Form Deformation (F3D) algorithm for non-rigid registration.
    Initially based on Modat et al., "Fast Free-Form Deformation using
    graphics processing units", CMPB, 2010

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegF3D()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.flo_file = 'im2.nii'
    >>> node.inputs.rmask_file = 'mask.nii'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_f3d -cpp im2_cpp.nii.gz -flo im2.nii -omp 4 -ref im1.nii \
-res im2_res.nii.gz -rmask mask.nii'

    """
    _cmd = get_custom_path('reg_f3d')
    input_spec = RegF3DInputSpec
    output_spec = RegF3DOutputSpec

    @staticmethod
    def _remove_extension(in_file):
        dn, bn, _ = split_filename(in_file)
        return os.path.join(dn, bn)

    def _list_outputs(self):
        outputs = super(RegF3D, self)._list_outputs()

        if self.inputs.vel_flag is True:
            res_name = self._remove_extension(outputs['res_file'])
            cpp_name = self._remove_extension(outputs['cpp_file'])
            outputs['invres_file'] = '%s_backward.nii.gz' % res_name
            outputs['invcpp_file'] = '%s_backward.nii.gz' % cpp_name

        # Make a list of the linear transformation file and the input image
        if self.inputs.vel_flag is True and isdefined(self.inputs.aff_file):
            cpp_file = os.path.abspath(outputs['cpp_file'])
            flo_file = os.path.abspath(self.inputs.flo_file)
            outputs['avg_output'] = '%s %s %s' % (self.inputs.aff_file,
                                                  cpp_file, flo_file)
        else:
            cpp_file = os.path.abspath(outputs['cpp_file'])
            flo_file = os.path.abspath(self.inputs.flo_file)
            outputs['avg_output'] = '%s %s' % (cpp_file, flo_file)

        return outputs



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
import os
import sys
import pytest
from nipype import config
from mock import MagicMock
from builtins import object

try:
    import xvfbwrapper
    has_Xvfb = True
except ImportError:
    has_Xvfb = False

# Define mocks for xvfbwrapper. Do not forget the spec to ensure that
# hasattr() checks return False with missing attributes.
xvfbpatch = MagicMock(spec=['Xvfb'])
xvfbpatch.Xvfb.return_value = MagicMock(
    spec=['new_display', 'start', 'stop'], new_display=2010)

# Mock the legacy xvfbwrapper.Xvfb class (changed display attribute name)
xvfbpatch_old = MagicMock(spec=['Xvfb'])
xvfbpatch_old.Xvfb.return_value = MagicMock(
    spec=['vdisplay_num', 'start', 'stop'], vdisplay_num=2010)


@pytest.mark.parametrize('dispvar', [':12', 'localhost:12', 'localhost:12.1'])
def test_display_parse(monkeypatch, dispvar):
    """Check that when $DISPLAY is defined, the display is correctly parsed"""
    config._display = None
    config._config.remove_option('execution', 'display_variable')
    monkeypatch.setenv('DISPLAY', dispvar)
    assert config.get_display() == ':12'
    # Test that it was correctly cached
    assert config.get_display() == ':12'


@pytest.mark.parametrize('dispnum', range(5))
def test_display_config(monkeypatch, dispnum):
    """Check that the display_variable option is used ($DISPLAY not set)"""
    config._display = None
    dispstr = ':%d' % dispnum
    config.set('execution', 'display_variable', dispstr)
    monkeypatch.delitem(os.environ, 'DISPLAY', raising=False)
    assert config.get_display() == config.get('execution', 'display_variable')
    # Test that it was correctly cached
    assert config.get_display() == config.get('execution', 'display_variable')


@pytest.mark.parametrize('dispnum', range(5))
def test_display_system(monkeypatch, dispnum):
    """Check that when only a $DISPLAY is defined, it is used"""
    config._display = None
    config._config.remove_option('execution', 'display_variable')
    dispstr = ':%d' % dispnum
    monkeypatch.setenv('DISPLAY', dispstr)
    assert config.get_display() == dispstr
    # Test that it was correctly cached
    assert config.get_display() == dispstr


def test_display_config_and_system(monkeypatch):
    """Check that when only both config and $DISPLAY are defined, the config
    takes precedence"""
    config._display = None
    dispstr = ':10'
    config.set('execution', 'display_variable', dispstr)
    monkeypatch.setenv('DISPLAY', ':0')
    assert config.get_display() == dispstr
    # Test that it was correctly cached
    assert config.get_display() == dispstr


def test_display_noconfig_nosystem_patched(monkeypatch):
    """Check that when no $DISPLAY nor option are specified, a virtual Xvfb is
    used"""
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.delitem(os.environ, 'DISPLAY', raising=False)
    monkeypatch.setitem(sys.modules, 'xvfbwrapper', xvfbpatch)
    monkeypatch.setattr(sys, 'platform', value='linux')
    assert config.get_display() == ":2010"
    # Test that it was correctly cached
    assert config.get_display() == ':2010'

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, 'platform', value='darwin')
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_empty_patched(monkeypatch):
    """
    Check that when $DISPLAY is empty string and no option is specified,
    a virtual Xvfb is used
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.setenv('DISPLAY', '')
    monkeypatch.setitem(sys.modules, 'xvfbwrapper', xvfbpatch)
    monkeypatch.setattr(sys, 'platform', value='linux')
    assert config.get_display() == ':2010'
    # Test that it was correctly cached
    assert config.get_display() == ':2010'

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, 'platform', value='darwin')
    with pytest.raises(RuntimeError):
        config.get_display()

def test_display_noconfig_nosystem_patched_oldxvfbwrapper(monkeypatch):
    """
    Check that when no $DISPLAY nor option are specified,
    a virtual Xvfb is used (with a legacy version of xvfbwrapper).
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.delitem(os.environ, 'DISPLAY', raising=False)
    monkeypatch.setitem(sys.modules, 'xvfbwrapper', xvfbpatch_old)
    monkeypatch.setattr(sys, 'platform', value='linux')
    assert config.get_display() == ":2010"
    # Test that it was correctly cached
    assert config.get_display() == ':2010'

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, 'platform', value='darwin')
    with pytest.raises(RuntimeError):
        config.get_display()

def test_display_empty_patched_oldxvfbwrapper(monkeypatch):
    """
    Check that when $DISPLAY is empty string and no option is specified,
    a virtual Xvfb is used (with a legacy version of xvfbwrapper).
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.setenv('DISPLAY', '')
    monkeypatch.setitem(sys.modules, 'xvfbwrapper', xvfbpatch_old)
    monkeypatch.setattr(sys, 'platform', value='linux')
    assert config.get_display() == ':2010'
    # Test that it was correctly cached
    assert config.get_display() == ':2010'

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, 'platform', value='darwin')
    with pytest.raises(RuntimeError):
        config.get_display()

def test_display_noconfig_nosystem_notinstalled(monkeypatch):
    """
    Check that an exception is raised if xvfbwrapper is not installed
    but necessary (no config and $DISPLAY unset)
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.delenv('DISPLAY', raising=False)
    monkeypatch.setitem(sys.modules, 'xvfbwrapper', None)
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_empty_notinstalled(monkeypatch):
    """
    Check that an exception is raised if xvfbwrapper is not installed
    but necessary (no config and $DISPLAY empty)
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.setenv('DISPLAY', '')
    monkeypatch.setitem(sys.modules, 'xvfbwrapper', None)
    with pytest.raises(RuntimeError):
        config.get_display()


@pytest.mark.skipif(not has_Xvfb, reason='xvfbwrapper not installed')
@pytest.mark.skipif('darwin' in sys.platform, reason='macosx requires root for Xvfb')
def test_display_noconfig_nosystem_installed(monkeypatch):
    """
    Check that actually uses xvfbwrapper when installed (not mocked)
    and necessary (no config and $DISPLAY unset)
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.delenv('DISPLAY', raising=False)
    newdisp = config.get_display()
    assert int(newdisp.split(':')[-1]) > 1000
    # Test that it was correctly cached
    assert config.get_display() == newdisp


@pytest.mark.skipif(not has_Xvfb, reason='xvfbwrapper not installed')
@pytest.mark.skipif('darwin' in sys.platform, reason='macosx requires root for Xvfb')
def test_display_empty_installed(monkeypatch):
    """
    Check that actually uses xvfbwrapper when installed (not mocked)
    and necessary (no config and $DISPLAY empty)
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.setenv('DISPLAY', '')
    newdisp = config.get_display()
    assert int(newdisp.split(':')[-1]) > 1000
    # Test that it was correctly cached
    assert config.get_display() == newdisp


def test_display_empty_macosx(monkeypatch):
    """
    Check that an exception is raised if xvfbwrapper is necessary
    (no config and $DISPLAY unset) but platform is OSX. See
    https://github.com/nipy/nipype/issues/1400
    """
    config._display = None
    if config.has_option('execution', 'display_variable'):
        config._config.remove_option('execution', 'display_variable')
    monkeypatch.delenv('DISPLAY', '')

    monkeypatch.setattr(sys, 'platform', 'darwin')
    with pytest.raises(RuntimeError):
        config.get_display()


def test_cwd_cached(tmpdir):
    """Check that changing dirs does not change nipype's cwd"""
    oldcwd = config.cwd
    tmpdir.chdir()
    assert config.cwd == oldcwd


def test_debug_mode():
    from ... import logging

    sofc_config = config.get('execution', 'stop_on_first_crash')
    ruo_config = config.get('execution', 'remove_unnecessary_outputs')
    ki_config = config.get('execution', 'keep_inputs')
    wf_config = config.get('logging', 'workflow_level')
    if_config = config.get('logging', 'interface_level')
    ut_config = config.get('logging', 'utils_level')

    wf_level = logging.getLogger('nipype.workflow').level
    if_level = logging.getLogger('nipype.interface').level
    ut_level = logging.getLogger('nipype.utils').level

    config.enable_debug_mode()

    # Check config is updated and logging levels, too
    assert config.get('execution', 'stop_on_first_crash') == 'true'
    assert config.get('execution', 'remove_unnecessary_outputs') == 'false'
    assert config.get('execution', 'keep_inputs') == 'true'
    assert config.get('logging', 'workflow_level') == 'DEBUG'
    assert config.get('logging', 'interface_level') == 'DEBUG'
    assert config.get('logging', 'utils_level') == 'DEBUG'

    assert logging.getLogger('nipype.workflow').level == 10
    assert logging.getLogger('nipype.interface').level == 10
    assert logging.getLogger('nipype.utils').level == 10

    # Restore config and levels
    config.set('execution', 'stop_on_first_crash', sofc_config)
    config.set('execution', 'remove_unnecessary_outputs', ruo_config)
    config.set('execution', 'keep_inputs', ki_config)
    config.set('logging', 'workflow_level', wf_config)
    config.set('logging', 'interface_level', if_config)
    config.set('logging', 'utils_level', ut_config)
    logging.update_logging(config)

    assert config.get('execution', 'stop_on_first_crash') == sofc_config
    assert config.get('execution', 'remove_unnecessary_outputs') == ruo_config
    assert config.get('execution', 'keep_inputs') == ki_config
    assert config.get('logging', 'workflow_level') == wf_config
    assert config.get('logging', 'interface_level') == if_config
    assert config.get('logging', 'utils_level') == ut_config

    assert logging.getLogger('nipype.workflow').level == wf_level
    assert logging.getLogger('nipype.interface').level == if_level
    assert logging.getLogger('nipype.utils').level == ut_level



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..specialized import BRAINSROIAuto


def test_BRAINSROIAuto_inputs():
    input_map = dict(
        ROIAutoDilateSize=dict(argstr='--ROIAutoDilateSize %f', ),
        args=dict(argstr='%s', ),
        closingSize=dict(argstr='--closingSize %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        otsuPercentileThreshold=dict(argstr='--otsuPercentileThreshold %f', ),
        outputClippedVolumeROI=dict(
            argstr='--outputClippedVolumeROI %s',
            hash_files=False,
        ),
        outputROIMaskVolume=dict(
            argstr='--outputROIMaskVolume %s',
            hash_files=False,
        ),
        outputVolumePixelType=dict(argstr='--outputVolumePixelType %s', ),
        thresholdCorrectionFactor=dict(
            argstr='--thresholdCorrectionFactor %f', ),
    )
    inputs = BRAINSROIAuto.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSROIAuto_outputs():
    output_map = dict(
        outputClippedVolumeROI=dict(extensions=None, ),
        outputROIMaskVolume=dict(extensions=None, ),
    )
    outputs = BRAINSROIAuto.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..epi import EPIDeWarp


def test_EPIDeWarp_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cleanup=dict(argstr='--cleanup', ),
        dph_file=dict(
            argstr='--dph %s',
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi_file=dict(
            argstr='--epi %s',
            extensions=None,
        ),
        epidw=dict(
            argstr='--epidw %s',
            genfile=False,
        ),
        esp=dict(
            argstr='--esp %s',
            usedefault=True,
        ),
        exf_file=dict(
            argstr='--exf %s',
            extensions=None,
        ),
        exfdw=dict(
            argstr='--exfdw %s',
            genfile=True,
        ),
        mag_file=dict(
            argstr='--mag %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        nocleanup=dict(
            argstr='--nocleanup',
            usedefault=True,
        ),
        output_type=dict(),
        sigma=dict(
            argstr='--sigma %s',
            usedefault=True,
        ),
        tediff=dict(
            argstr='--tediff %s',
            usedefault=True,
        ),
        tmpdir=dict(
            argstr='--tmpdir %s',
            genfile=True,
        ),
        vsm=dict(
            argstr='--vsm %s',
            genfile=True,
        ),
    )
    inputs = EPIDeWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_EPIDeWarp_outputs():
    output_map = dict(
        exf_mask=dict(extensions=None, ),
        exfdw=dict(extensions=None, ),
        unwarped_file=dict(extensions=None, ),
        vsm_file=dict(extensions=None, ),
    )
    outputs = EPIDeWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brains import BRAINSConstellationModeler


def test_BRAINSConstellationModeler_inputs():
    input_map = dict(
        BackgroundFillValue=dict(argstr='--BackgroundFillValue %s', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputTrainingList=dict(
            argstr='--inputTrainingList %s',
            extensions=None,
        ),
        mspQualityLevel=dict(argstr='--mspQualityLevel %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        optimizedLandmarksFilenameExtender=dict(
            argstr='--optimizedLandmarksFilenameExtender %s', ),
        outputModel=dict(
            argstr='--outputModel %s',
            hash_files=False,
        ),
        rescaleIntensities=dict(argstr='--rescaleIntensities ', ),
        rescaleIntensitiesOutputRange=dict(
            argstr='--rescaleIntensitiesOutputRange %s',
            sep=',',
        ),
        resultsDir=dict(
            argstr='--resultsDir %s',
            hash_files=False,
        ),
        saveOptimizedLandmarks=dict(argstr='--saveOptimizedLandmarks ', ),
        trimRescaledIntensities=dict(argstr='--trimRescaledIntensities %f', ),
        verbose=dict(argstr='--verbose ', ),
        writedebuggingImagesLevel=dict(
            argstr='--writedebuggingImagesLevel %d', ),
    )
    inputs = BRAINSConstellationModeler.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSConstellationModeler_outputs():
    output_map = dict(
        outputModel=dict(extensions=None, ),
        resultsDir=dict(),
    )
    outputs = BRAINSConstellationModeler.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..mesh import P2PDistance


def test_P2PDistance_inputs():
    input_map = dict(
        metric=dict(usedefault=True, ),
        out_file=dict(
            extensions=None,
            usedefault=True,
        ),
        out_warp=dict(
            extensions=None,
            usedefault=True,
        ),
        surface1=dict(
            extensions=None,
            mandatory=True,
        ),
        surface2=dict(
            extensions=None,
            mandatory=True,
        ),
        weighting=dict(usedefault=True, ),
    )
    inputs = P2PDistance.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_P2PDistance_outputs():
    output_map = dict(
        distance=dict(),
        out_file=dict(extensions=None, ),
        out_warp=dict(extensions=None, ),
    )
    outputs = P2PDistance.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Tests for the engine workflows module
"""
from glob import glob
import os
from shutil import rmtree
from itertools import product
import pytest
import networkx as nx

from .... import config
from ....interfaces import utility as niu
from ... import engine as pe
from .test_base import EngineTestInterface
from .test_utils import UtilsTestInterface


def test_init():
    with pytest.raises(TypeError):
        pe.Workflow()
    pipe = pe.Workflow(name='pipe')
    assert type(pipe._graph) == nx.DiGraph


def test_connect():
    pipe = pe.Workflow(name='pipe')
    mod2 = pe.Node(EngineTestInterface(), name='mod2')
    mod1 = pe.Node(EngineTestInterface(), name='mod1')
    pipe.connect([(mod1, mod2, [('output1', 'input1')])])

    assert mod1 in pipe._graph.nodes()
    assert mod2 in pipe._graph.nodes()
    assert pipe._graph.get_edge_data(mod1, mod2) == {
        'connect': [('output1', 'input1')]
    }


def test_add_nodes():
    pipe = pe.Workflow(name='pipe')
    mod1 = pe.Node(EngineTestInterface(), name='mod1')
    mod2 = pe.Node(EngineTestInterface(), name='mod2')
    pipe.add_nodes([mod1, mod2])

    assert mod1 in pipe._graph.nodes()
    assert mod2 in pipe._graph.nodes()


def test_disconnect():
    a = pe.Node(niu.IdentityInterface(fields=['a', 'b']), name='a')
    b = pe.Node(niu.IdentityInterface(fields=['a', 'b']), name='b')
    flow1 = pe.Workflow(name='test')
    flow1.connect(a, 'a', b, 'a')
    flow1.disconnect(a, 'a', b, 'a')
    assert list(flow1._graph.edges()) == []


def test_workflow_add():
    n1 = pe.Node(niu.IdentityInterface(fields=['a', 'b']), name='n1')
    n2 = pe.Node(niu.IdentityInterface(fields=['c', 'd']), name='n2')
    n3 = pe.Node(niu.IdentityInterface(fields=['c', 'd']), name='n1')
    w1 = pe.Workflow(name='test')
    w1.connect(n1, 'a', n2, 'c')
    for node in [n1, n2, n3]:
        with pytest.raises(IOError):
            w1.add_nodes([node])
    with pytest.raises(IOError):
        w1.connect([(w1, n2, [('n1.a', 'd')])])


def test_doubleconnect():
    a = pe.Node(niu.IdentityInterface(fields=['a', 'b']), name='a')
    b = pe.Node(niu.IdentityInterface(fields=['a', 'b']), name='b')
    flow1 = pe.Workflow(name='test')
    flow1.connect(a, 'a', b, 'a')
    with pytest.raises(Exception) as excinfo:
        flow1.connect(a, 'b', b, 'a')
    assert "Trying to connect" in str(excinfo.value)

    c = pe.Node(niu.IdentityInterface(fields=['a', 'b']), name='c')
    flow1 = pe.Workflow(name='test2')
    with pytest.raises(Exception) as excinfo:
        flow1.connect([(a, c, [('b', 'b')]), (b, c, [('a', 'b')])])
    assert "Trying to connect" in str(excinfo.value)


def test_duplicate_node_check():

    wf = pe.Workflow(name="testidentity")

    original_list = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    selector1 = pe.Node(niu.Select(), name="selector1")
    selector1.inputs.index = original_list[:-1]
    selector1.inputs.inlist = original_list
    selector2 = pe.Node(niu.Select(), name="selector2")
    selector2.inputs.index = original_list[:-2]
    selector3 = pe.Node(niu.Select(), name="selector3")
    selector3.inputs.index = original_list[:-3]
    selector4 = pe.Node(niu.Select(), name="selector3")
    selector4.inputs.index = original_list[:-4]

    wf_connections = [
        (selector1, selector2, [("out", "inlist")]),
        (selector2, selector3, [("out", "inlist")]),
        (selector3, selector4, [("out", "inlist")]),
    ]

    with pytest.raises(IOError) as excinfo:
        wf.connect(wf_connections)
    assert 'Duplicate node name "selector3" found.' == str(excinfo.value)


def _test_function(arg1):
    import os
    file1 = os.path.join(os.getcwd(), 'file1.txt')
    file2 = os.path.join(os.getcwd(), 'file2.txt')
    file3 = os.path.join(os.getcwd(), 'file3.txt')
    file4 = os.path.join(os.getcwd(), 'subdir', 'file4.txt')
    os.mkdir("subdir")
    for filename in [file1, file2, file3, file4]:
        with open(filename, 'wt') as fp:
            fp.write('%d' % arg1)
    return file1, file2, os.path.join(os.getcwd(), "subdir")


def _test_function2(in_file, arg):
    import os
    with open(in_file, 'rt') as fp:
        in_arg = fp.read()

    file1 = os.path.join(os.getcwd(), 'file1.txt')
    file2 = os.path.join(os.getcwd(), 'file2.txt')
    file3 = os.path.join(os.getcwd(), 'file3.txt')
    files = [file1, file2, file3]
    for filename in files:
        with open(filename, 'wt') as fp:
            fp.write('%d' % arg + in_arg)
    return file1, file2, 1


def _test_function3(arg):
    return arg


@pytest.mark.parametrize(
    'plugin, remove_unnecessary_outputs, keep_inputs',
    list(product(['Linear', 'MultiProc'], [False, True], [True, False])))
def test_outputs_removal_wf(tmpdir, plugin, remove_unnecessary_outputs,
                            keep_inputs):
    config.set_default_config()
    config.set('execution', 'remove_unnecessary_outputs',
               remove_unnecessary_outputs)
    config.set('execution', 'keep_inputs', keep_inputs)

    n1 = pe.Node(
        niu.Function(
            output_names=['out_file1', 'out_file2', 'dir'],
            function=_test_function),
        name='n1',
        base_dir=tmpdir.strpath)
    n1.inputs.arg1 = 1

    n2 = pe.Node(
        niu.Function(
            output_names=['out_file1', 'out_file2', 'n'],
            function=_test_function2),
        name='n2',
        base_dir=tmpdir.strpath)
    n2.inputs.arg = 2

    n3 = pe.Node(
        niu.Function(
            output_names=['n'],
            function=_test_function3),
        name='n3',
        base_dir=tmpdir.strpath)

    wf = pe.Workflow(
        name="node_rem_test" + plugin, base_dir=tmpdir.strpath)

    wf.connect(n1, "out_file1", n2, "in_file")
    wf.run(plugin=plugin)

    # Necessary outputs HAVE to exist
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n1.name, 'file1.txt'))
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n2.name, 'file1.txt'))
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n2.name, 'file2.txt'))

    # Unnecessary outputs exist only iff remove_unnecessary_outputs is True
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n1.name,
                     'file2.txt')) is not remove_unnecessary_outputs
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n1.name, "subdir",
                     'file4.txt')) is not remove_unnecessary_outputs
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n1.name,
                     'file3.txt')) is not remove_unnecessary_outputs
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n2.name,
                     'file3.txt')) is not remove_unnecessary_outputs

    n4 = pe.Node(UtilsTestInterface(), name='n4', base_dir=tmpdir.strpath)
    wf.connect(n2, "out_file1", n4, "in_file")

    def pick_first(l):
        return l[0]

    wf.connect(n4, ("output1", pick_first), n3, "arg")
    rmtree(os.path.join(wf.base_dir, wf.name))
    wf.run(plugin=plugin)

    # Test necessary outputs
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n2.name, 'file1.txt'))
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n2.name, 'file1.txt'))

    # Test unnecessary outputs
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n2.name,
                     'file2.txt')) is not remove_unnecessary_outputs

    # Test keep_inputs
    assert os.path.exists(
        os.path.join(wf.base_dir, wf.name, n4.name,
                     'file1.txt')) is keep_inputs


def _test_function4():
    raise FileNotFoundError('Generic error')


def test_config_setting(tmpdir):
    tmpdir.chdir()
    wf = pe.Workflow('config')
    wf.base_dir = os.getcwd()

    crashdir = os.path.join(os.getcwd(), 'crashdir')
    os.mkdir(crashdir)
    wf.config = {"execution": {"crashdump_dir": crashdir}}

    n1 = pe.Node(niu.Function(function=_test_function4),
                 name='errorfunc')
    wf.add_nodes([n1])
    try:
        wf.run()
    except RuntimeError:
        pass

    fl = glob(os.path.join(crashdir, 'crash*'))
    assert len(fl) == 1

    # Now test node overwrite
    crashdir2 = os.path.join(os.getcwd(), 'crashdir2')
    os.mkdir(crashdir2)
    crashdir3 = os.path.join(os.getcwd(), 'crashdir3')
    os.mkdir(crashdir3)
    wf.config = {"execution": {"crashdump_dir": crashdir3}}
    n1.config = {"execution": {"crashdump_dir": crashdir2}}

    try:
        wf.run()
    except RuntimeError:
        pass

    fl = glob(os.path.join(crashdir2, 'crash*'))
    assert len(fl) == 1
    fl = glob(os.path.join(crashdir3, 'crash*'))
    assert len(fl) == 0
